@@ -1,3 +1,4 @@
+
 /*
  * bltBgexec.c --
  *
@@ -31,18 +32,21 @@
 #include <signal.h>
 
 #ifndef BGEXEC_VERSION
-#define BGEXEC_VERSION "1.2"
+#define BGEXEC_VERSION "1.3"
 #endif
 
 #define BUFFER_SIZE	1000	/* Maximum number of bytes per read */
-#define MAX_READS       100	/* Maximum number of successful reads before
-			         * stopping to let Tk catch up on events */
+#define MAX_READS       100	/* Maximum number of successful reads
+			         * before stopping to let Tk catch up
+			         * on events */
 
 typedef struct {
-    char *storage;		/* Buffer to store command output (malloc-ed):
-				 * Initially points to static storage */
-    int used;			/* Number of characters read into the buffer */
-    int size;			/* Size of buffer allocated */
+    char *storage;		/* Buffer to store command output
+				 * (malloc-ed): Initially points to
+				 * static storage */
+    unsigned int used;		/* Number of characters read into the
+				 * buffer */
+    unsigned int size;		/* Size of buffer allocated */
     char staticSpace[BUFFER_SIZE * 2 + 1];	/* Static buffer space */
 
 } Buffer;
@@ -50,38 +54,46 @@ typedef struct {
 typedef struct {
     Tcl_Interp *interp;		/* Interpreter containing variable */
 
-    char *updateName;		/* Name of a Tcl variable (malloc'ed) to be
-				 * updated when no more data is currently
-				 * available for reading from the output pipe.
-				 * It's appended with the contents of the
-				 * current buffer (data which has arrived
-				 * since the last idle point). If it's NULL,
-				 * no updates are made */
+    char *updateName;		/* Name of a Tcl variable (malloc'ed)
+				 * to be updated when no more data is
+				 * currently available for reading
+				 * from the output pipe.  It's
+				 * appended with the contents of the
+				 * current buffer (data which has
+				 * arrived since the last idle
+				 * point). If it's NULL, no updates
+				 * are made */
 
-    char *outputName;		/* Name of a Tcl variable (malloc'ed) to be
-				 * set with the contents of stdout after the
-				 * last UNIX subprocess has completed. Setting
-				 * this variable triggers the termination of
-				 * all subprocesses, regardless whether they
-				 * have already completed or not */
+    char *outputName;		/* Name of a Tcl variable (malloc'ed)
+				 * to be set with the contents of
+				 * stdout after the last UNIX
+				 * subprocess has completed. Setting
+				 * this variable triggers the
+				 * termination of all subprocesses,
+				 * regardless whether they have
+				 * already completed or not */
 
-    char *errorName;		/* Name of a Tcl variable (malloc'ed) to hold
-				 * any available data from standard error */
+    char *errorName;		/* Name of a Tcl variable (malloc'ed)
+				 * to hold any available data from
+				 * standard error */
 
     int outputId;		/* File descriptor for output pipe.  */
     int errorId;		/* File Descriptor for error file. */
 
     Buffer buffer;		/* Buffer storing subprocess' stdin/stderr */
 
-    int numPids;		/* Number of processes created in pipeline */
+    unsigned int numPids;	/* Number of processes created in pipeline */
     int *pidPtr;		/* Array of process Ids. */
 
-    int keepFlag;		/* Indicates to set Tcl output variables with
-				 * trailing newlines intact */
-    int lastCount;		/* Number of bytes read the last time a
+    unsigned int sigNum;	/* If non-zero, indicates signal to send
+				 * subprocesses when cleaning up.*/
+    int keepFlag;		/* Indicates to set Tcl output
+				 * variables with trailing newlines
+				 * intact */
+    unsigned int lastCount;	/* Number of bytes read the last time a
 				 * buffer was retrieved */
-    int fixMark;		/* Index of fixed newline character in buffer.
-				 * If -1, no fix was made. */
+    int fixMark;		/* Index of fixed newline character in
+				 * buffer.  If -1, no fix was made. */
 
 } BackgroundInfo;
 
@@ -218,13 +230,12 @@ AppendOutputToBuffer(f, bufferPtr)
      * Worry about indefinite postponement.
      *
      * Typically we want to stay in the read loop as long as it takes
-     * to collect up all the data that's currently available.  But if it's
-     * coming in at a constant high rate, we need to arbitrarily break out
-     * at some point. This allows for both setting the output variable and
-     * the Tk program to handle idle events.
+     * to collect all the data that's currently available.  But if
+     * it's coming in at a constant high rate, we need to arbitrarily
+     * break out at some point. This allows for both setting the
+     * output variable and the Tk program to handle idle events.
      *
-     * -------------------------------------------------------------------
-     */
+     * ------------------------------------------------------------------- */
 
     for (i = 0; i < MAX_READS; i++) {
 
@@ -338,6 +349,52 @@ GetLastAppended(infoPtr)
     return (string);
 }
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * DestroyBackgroundInfo --
+ *
+ * 	This procedure is invoked by Tk_EventuallyFree or Tk_Release
+ *	to clean up the internal structure (BackgroundInfo) at a safe
+ *	time (when no-one is using it anymore).
+ *
+ *	Right now, our only concern is protecting infoPtr->outputName,
+ *	since subsequent calls to trace procedures (via CallTraces) may
+ *	still use it (as part1 and possibly part2).
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	The memory allocated to the BackgroundInfo structure released.
+ *
+ *----------------------------------------------------------------------
+ */
+
+ /* ARGSUSED */
+static void
+DestroyBackgroundInfo(clientData)
+    ClientData clientData;	/* Background info record. */
+{
+    BackgroundInfo *infoPtr = (BackgroundInfo *)clientData;
+
+    ResetBuffer(&(infoPtr->buffer));
+    if (infoPtr->updateName != NULL) {
+	free(infoPtr->updateName);
+    }
+    if (infoPtr->outputName != NULL) {
+	free(infoPtr->outputName);
+    }
+    if (infoPtr->errorName != NULL) {
+	free(infoPtr->errorName);
+    }
+    if (infoPtr->pidPtr != NULL) {
+	free((char *)infoPtr->pidPtr);
+	Tcl_ReapDetachedProcs();
+    }
+    free((char *)infoPtr);
+}
+
 /*
  *----------------------------------------------------------------------
  *
@@ -366,23 +423,9 @@ FakeOutputProc(clientData)
 {
     BackgroundInfo *infoPtr = (BackgroundInfo *)clientData;
 
-    Tcl_SetVar2(infoPtr->interp, infoPtr->outputName, (char *)NULL,
+    Tcl_SetVar(infoPtr->interp, infoPtr->outputName,
 	GetBuffer(&(infoPtr->buffer)), TCL_GLOBAL_ONLY);
-
-    if (infoPtr->updateName != NULL) {
-	free(infoPtr->updateName);
-    }
-    if (infoPtr->outputName != NULL) {
-	free(infoPtr->outputName);
-    }
-    if (infoPtr->errorName != NULL) {
-	free(infoPtr->errorName);
-    }
-    if (infoPtr->pidPtr != NULL) {
-	free((char *)infoPtr->pidPtr);
-	Tcl_ReapDetachedProcs();
-    }
-    free((char *)infoPtr);
+    DestroyBackgroundInfo(clientData);
 }
 
 /*
@@ -402,8 +445,7 @@ FakeOutputProc(clientData)
  *	Always returns NULL.
  *
  * Side effects:
- *	The output stream is closed, the variable trace is deleted,
- *	and memory allocated to the BackgroundInfo structure released.
+ *	The output stream is closed, the variable trace is deleted.
  *	In addition, the subprocesses are signaled for termination.
  *
  *----------------------------------------------------------------------
@@ -445,7 +487,7 @@ CleanupProc(clientData, interp, part1, part2, flags)
 		if (!infoPtr->keepFlag) {
 		    FixNewline(infoPtr);
 		}
-		Tcl_SetVar2(infoPtr->interp, infoPtr->errorName, (char *)NULL,
+		Tcl_SetVar(infoPtr->interp, infoPtr->errorName,
 		    GetBuffer(&(infoPtr->buffer)), TCL_GLOBAL_ONLY);
 	    } else if (result == TCL_ERROR) {
 		Tcl_AppendResult(infoPtr->interp, "error appending buffer: ",
@@ -458,27 +500,14 @@ CleanupProc(clientData, interp, part1, part2, flags)
     Tk_DeleteFileHandler(infoPtr->outputId);
     Tcl_UntraceVar2(interp, part1, part2, flags, CleanupProc, clientData);
 
-    if (infoPtr->updateName != NULL) {
-	free(infoPtr->updateName);
-    }
-    if (infoPtr->errorName != NULL) {
-	free(infoPtr->errorName);
-    }
-    if (infoPtr->outputName != NULL) {
-	free(infoPtr->outputName);
-    }
-    ResetBuffer(&(infoPtr->buffer));
-
-    if (infoPtr->pidPtr != NULL) {
+    if ((infoPtr->pidPtr != NULL) && (infoPtr->sigNum > 0)) {
 	register int i;
 
 	for (i = 0; i < infoPtr->numPids; i++) {
-	    kill(infoPtr->pidPtr[i], 1);
+	    kill(infoPtr->pidPtr[i], (int)infoPtr->sigNum);
 	}
-	free((char *)infoPtr->pidPtr);
-	Tcl_ReapDetachedProcs();
     }
-    free((char *)infoPtr);
+    Tk_EventuallyFree((ClientData)infoPtr, DestroyBackgroundInfo);
     return NULL;
 }
 
@@ -517,7 +546,7 @@ BackgroundProc(clientData, mask)
 	    if (!infoPtr->keepFlag) {
 		FixNewline(infoPtr);
 	    }
-	    Tcl_SetVar2(infoPtr->interp, infoPtr->updateName, (char *)NULL,
+	    Tcl_SetVar(infoPtr->interp, infoPtr->updateName,
 		GetLastAppended(infoPtr), (TCL_GLOBAL_ONLY | TCL_APPEND_VALUE));
 	    if (!infoPtr->keepFlag) {
 		UnfixNewline(infoPtr);
@@ -538,12 +567,13 @@ BackgroundProc(clientData, mask)
 	FixNewline(infoPtr);
     }
     if (infoPtr->updateName != NULL) {
-	Tcl_SetVar2(infoPtr->interp, infoPtr->updateName, (char *)NULL,
+	Tcl_SetVar(infoPtr->interp, infoPtr->updateName,
 	    GetLastAppended(infoPtr), (TCL_GLOBAL_ONLY | TCL_APPEND_VALUE));
     }
-    Tcl_SetVar2(infoPtr->interp, infoPtr->outputName, (char *)NULL,
+    Tk_Preserve((ClientData)infoPtr);
+    Tcl_SetVar(infoPtr->interp, infoPtr->outputName,
 	GetBuffer(&(infoPtr->buffer)), TCL_GLOBAL_ONLY);
-
+    Tk_Release((ClientData)infoPtr);
 }
 
 /*
@@ -593,6 +623,7 @@ BgExecCmd(clientData, interp, argc, argv)
     }
     infoPtr->interp = interp;
     infoPtr->keepFlag = 0;
+    infoPtr->sigNum = 1;	/* SIGHUP */
     infoPtr->errorName = infoPtr->updateName = infoPtr->outputName = NULL;
     InitBuffer(&(infoPtr->buffer));
 
@@ -601,45 +632,54 @@ BgExecCmd(clientData, interp, argc, argv)
 
     for (i = 1; i < argc; i++) {
 	if ((parseSwitches) && (argv[i][0] == '-')) {
-	    int length = strlen(argv[i]);
-	    char c = argv[i][1];
+	    register char *swtch;
+	    int length;
+	    char c;
 
-	    if ((c == 'u') && (strncmp(argv[i], "-updatevar", length) == 0)) {
-		i++;
-		if (i == argc) {
-		    Tcl_AppendResult(interp, "no variable for \"-updatevar\"",
-			(char *)NULL);
-		    goto error;
-		}
+	    swtch = argv[i];
+	    length = strlen(swtch);
+	    c = swtch[1];
+
+	    if ((c == '-') && (swtch[2] == '\0')) {
+		parseSwitches = 0;
+		continue;
+	    } else if ((c == 'k') && (length > 1) &&
+		(strncmp(swtch, "-keepnewline", length) == 0)) {
+		infoPtr->keepFlag = 1;
+		continue;
+	    }
+	    i++;
+
+	    if (i == argc) {
+		Tcl_AppendResult(interp, "missing value for \"", swtch, "\"",
+		    (char *)NULL);
+		goto error;
+	    }
+	    if ((c == 'u') && (strncmp(swtch, "-updatevar", length) == 0)) {
 		infoPtr->updateName = strdup(argv[i]);
 	    } else if ((c == 'e') &&
-		(strncmp(argv[i], "-errorvar", length) == 0)) {
-		i++;
-		if (i == argc) {
-		    Tcl_AppendResult(interp, "no variable for \"-errorvar\"",
-			(char *)NULL);
-		    goto error;
-		}
+		(strncmp(swtch, "-errorvar", length) == 0)) {
 		infoPtr->errorName = strdup(argv[i]);
 		errFilePtr = &errorId;
 	    } else if ((c == 'o') &&
-		(strncmp(argv[i], "-outputvar", length) == 0)) {
-		i++;
-		if (i == argc) {
-		    Tcl_AppendResult(interp, "no variable for \"-outputvar\"",
-			(char *)NULL);
+		(strncmp(swtch, "-outputvar", length) == 0)) {
+		infoPtr->outputName = strdup(argv[i]);
+	    } else if ((c == 'k') && (length > 1) &&
+		(strncmp(swtch, "-killsignal", length) == 0)) {
+		int value;
+
+		if (Tcl_GetInt(interp, argv[i], &value) != TCL_OK) {
 		    goto error;
 		}
-		infoPtr->outputName = strdup(argv[i]);
-	    } else if ((c == 'k') &&
-		(strncmp(argv[i], "-keepnewline", length) == 0)) {
-		infoPtr->keepFlag = 1;
-	    } else if ((c == '-') && (argv[i][2] == '\0')) {
-		parseSwitches = 0;
+		if ((value < 0) || (value > 31)) {
+		    Tcl_AppendResult(interp, "bad kill signal number \"",
+			argv[i], "\"", (char *)NULL);
+		}
+		infoPtr->sigNum = (unsigned int)value;
 	    } else {
-		Tcl_AppendResult(interp, "bad switch \"", argv[i], "\": ",
-		    "should be -errorvar, -keepnewline, -outputvar, ",
-		    "-updatevar, or --", (char *)NULL);
+		Tcl_AppendResult(interp, "bad switch \"", swtch, "\": ",
+		    "should be -errorvar, -keepnewline, -killsignal, ",
+		    "-outputvar, -updatevar, or --", (char *)NULL);
 		goto error;
 	    }
 	} else {
@@ -661,7 +701,7 @@ BgExecCmd(clientData, interp, argc, argv)
     }
     infoPtr->outputId = outputId;
     infoPtr->errorId = errorId;
-    infoPtr->numPids = numPids;
+    infoPtr->numPids = (unsigned int)numPids;
     infoPtr->pidPtr = pidPtr;
     infoPtr->lastCount = 0;
     infoPtr->fixMark = -1;
@@ -691,7 +731,7 @@ BgExecCmd(clientData, interp, argc, argv)
 
 	Tk_CreateFileHandler(outputId, TK_READABLE, BackgroundProc,
 	    (ClientData)infoPtr);
-	Tcl_TraceVar2(interp, infoPtr->outputName, (char *)NULL,
+	Tcl_TraceVar(interp, infoPtr->outputName,
 	    (TCL_TRACE_WRITES | TCL_GLOBAL_ONLY), CleanupProc,
 	    (ClientData)infoPtr);
     }
