@@ -1,3 +1,4 @@
+
 /*
  * bltGrAxis.c --
  *
@@ -43,7 +44,7 @@
 /*
  * Round x in terms of units
  */
-#define UROUND(x,u)	(ROUND((x)/(u))*(u))
+#define UROUND(x,u)	(BLT_RND((x)/(u))*(u))
 #define UCEIL(x,u)	(ceil((x)/(u))*(u))
 #define UFLOOR(x,u)	(floor((x)/(u))*(u))
 
@@ -61,12 +62,12 @@ enum LimitIndices {
 /* Note: The following array is ordered according to enum AxisTypes */
 static char *axisNames[] =
 {
-    "x", "x2", "y", "y2"
+    "x", "y", "x2", "y2"
 };
 
 /* Map normalized coordinates to window coordinates */
-#define MAPX(X,x)    	(ROUND((x)*(X)->scale)+(X)->offset)
-#define MAPY(Y,y)    	((Y)->offset-ROUND((y)*(Y)->scale))
+#define MAPX(X,x)    	(BLT_RND((x)*(X)->scale)+(X)->offset)
+#define MAPY(Y,y)    	((Y)->offset-BLT_RND((y)*(Y)->scale))
 
 /* Map graph coordinates to normalized coordinates [0..1] */
 #define NORM(a,x) 	(((x) - (a)->min) / (a)->range)
@@ -220,16 +221,25 @@ typedef struct {
 #define VERTICAL_AXIS(a)	((a)->location&1)
 #define HORIZONTAL_AXIS(a)    	(!((a)->location&1))
 
+#define DEF_AXIS_ALT_MAPPED 	"false"
+#define DEF_AXIS_COMMAND	(char *)NULL
+#define DEF_AXIS_DESCENDING 	"0"
 #define DEF_AXIS_FG_COLOR	BLACK
 #define DEF_AXIS_FG_MONO	BLACK
 #define DEF_AXIS_FONT	 	"*-Courier-Bold-R-Normal-*-100-*"
+#define DEF_AXIS_LINE_WIDTH	"0"
+#define DEF_AXIS_LOG_SCALE 	"0"
+#define DEF_AXIS_LOOSE 		"0"
+#define DEF_AXIS_MAX		(char *)NULL
+#define DEF_AXIS_MIN		(char *)NULL
+#define DEF_AXIS_ROTATE		"0.0"
+#define DEF_AXIS_STD_MAPPED 	"1"
+#define DEF_AXIS_STEPSIZE	"0.0"
+#define DEF_AXIS_SUBTICKS	"2"
 #define DEF_AXIS_TICKS		"1"
-#define DEF_AXIS_SUBTICKS_GRAPH	"2"
-#define DEF_AXIS_X_SUBTICKS_BARCHART	"0"
 #define DEF_AXIS_X_STEPSIZE_BARCHART	"1.0"
-#define DEF_AXIS_ALT_MAPPED 	"false"
-#define DEF_AXIS_STD_MAPPED 	"true"
-#define DEF_AXIS_DESCENDING 	"false"
+#define DEF_AXIS_X_SUBTICKS_BARCHART	"0"
+#define DEF_AXIS_TITLE		(char *)NULL
 
 static Tk_ConfigSpec xAxisConfigSpecs[] =
 {
@@ -240,7 +250,7 @@ static Tk_ConfigSpec xAxisConfigSpecs[] =
 	DEF_AXIS_FG_MONO, Tk_Offset(Axis, fgColorPtr),
 	TK_CONFIG_MONO_ONLY | ALL_MASK},
     {TK_CONFIG_STRING, "-command", "xCommand", "AxisCommand",
-	(char *)NULL, Tk_Offset(Axis, formatCmd),
+	DEF_AXIS_COMMAND, Tk_Offset(Axis, formatCmd),
 	ALL_MASK | TK_CONFIG_NULL_OK},
     {TK_CONFIG_BOOLEAN, "-descending", "xDescending", "AxisDescending",
 	DEF_AXIS_DESCENDING, Tk_Offset(Axis, descending),
@@ -248,30 +258,34 @@ static Tk_ConfigSpec xAxisConfigSpecs[] =
     {TK_CONFIG_FONT, "-font", "xFont", "AxisFont",
 	DEF_AXIS_FONT, Tk_Offset(Axis, fontPtr), ALL_MASK},
     {TK_CONFIG_PIXELS, "-linewidth", "xLinewidth", "AxisLinewidth",
-	(char *)NULL, Tk_Offset(Axis, lineWidth), ALL_MASK},
+	DEF_AXIS_LINE_WIDTH, Tk_Offset(Axis, lineWidth),
+	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_BOOLEAN, "-logscale", "xLogscale", "AxisLogscale",
-	(char *)NULL, Tk_Offset(Axis, logScale), ALL_MASK},
+	DEF_AXIS_LOG_SCALE, Tk_Offset(Axis, logScale),
+	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_BOOLEAN, "-loose", "xLoose", "AxisLoose",
-	(char *)NULL, Tk_Offset(Axis, loose), ALL_MASK},
+	DEF_AXIS_LOOSE, Tk_Offset(Axis, loose),
+	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_BOOLEAN, "-mapped", "xMapped", "AxisMapped",
 	DEF_AXIS_STD_MAPPED, Tk_Offset(Axis, mapped),
 	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_CUSTOM, "-max", "xMax", "AxisMax",
-	(char *)NULL, 0, ALL_MASK | TK_CONFIG_NULL_OK, &MaxLimitOption},
+	DEF_AXIS_MAX, 0, ALL_MASK | TK_CONFIG_NULL_OK, &MaxLimitOption},
     {TK_CONFIG_CUSTOM, "-min", "xMin", "AxisMin",
-	(char *)NULL, 0, ALL_MASK | TK_CONFIG_NULL_OK, &MinLimitOption},
+	DEF_AXIS_MIN, 0, ALL_MASK | TK_CONFIG_NULL_OK, &MinLimitOption},
     {TK_CONFIG_DOUBLE, "-rotate", "xRotate", "AxisRotate",
-	(char *)NULL, Tk_Offset(Axis, theta), ALL_MASK},
+	DEF_AXIS_ROTATE, Tk_Offset(Axis, theta),
+	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_BOOLEAN, "-showticks", "xShowticks", "AxisShowticks",
-	DEF_AXIS_TICKS, Tk_Offset(Axis, showTicks), ALL_MASK},
+	DEF_AXIS_TICKS, Tk_Offset(Axis, showTicks),
+	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_DOUBLE, "-stepsize", "xStepsize", "AxisStepsize",
-	(char *)NULL, Tk_Offset(Axis, reqStep), XYGRAPH_MASK},
+	DEF_AXIS_STEPSIZE, Tk_Offset(Axis, reqStep), XYGRAPH_MASK},
     {TK_CONFIG_DOUBLE, "-stepsize", "xStepsize", "AxisStepsize",
 	DEF_AXIS_X_STEPSIZE_BARCHART, Tk_Offset(Axis, reqStep),
 	BARCHART_MASK},
     {TK_CONFIG_INT, "-subticks", "xSubticks", "AxisSubticks",
-	DEF_AXIS_SUBTICKS_GRAPH, Tk_Offset(Axis, reqSubTicks),
-	XYGRAPH_MASK},
+	DEF_AXIS_SUBTICKS, Tk_Offset(Axis, reqSubTicks), XYGRAPH_MASK},
     {TK_CONFIG_INT, "-subticks", "xSubticks", "AxisSubticks",
 	DEF_AXIS_X_SUBTICKS_BARCHART, Tk_Offset(Axis, reqSubTicks),
 	BARCHART_MASK},
@@ -290,7 +304,7 @@ static Tk_ConfigSpec x2AxisConfigSpecs[] =
 	DEF_AXIS_FG_MONO, Tk_Offset(Axis, fgColorPtr),
 	TK_CONFIG_MONO_ONLY | ALL_MASK},
     {TK_CONFIG_STRING, "-command", "x2Command", "AxisCommand",
-	(char *)NULL, Tk_Offset(Axis, formatCmd),
+	DEF_AXIS_COMMAND, Tk_Offset(Axis, formatCmd),
 	ALL_MASK | TK_CONFIG_NULL_OK},
     {TK_CONFIG_BOOLEAN, "-descending", "x2Descending", "AxisDescending",
 	DEF_AXIS_DESCENDING, Tk_Offset(Axis, descending),
@@ -298,28 +312,34 @@ static Tk_ConfigSpec x2AxisConfigSpecs[] =
     {TK_CONFIG_FONT, "-font", "x2Font", "AxisFont",
 	DEF_AXIS_FONT, Tk_Offset(Axis, fontPtr), ALL_MASK},
     {TK_CONFIG_PIXELS, "-linewidth", "x2Linewidth", "AxisLinewidth",
-	(char *)NULL, Tk_Offset(Axis, lineWidth), ALL_MASK},
+	DEF_AXIS_LINE_WIDTH, Tk_Offset(Axis, lineWidth),
+	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_BOOLEAN, "-logscale", "x2Logscale", "AxisLogscale",
-	(char *)NULL, Tk_Offset(Axis, logScale), ALL_MASK},
+	DEF_AXIS_LOG_SCALE, Tk_Offset(Axis, logScale),
+	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_BOOLEAN, "-loose", "x2Loose", "AxisLoose",
-	(char *)NULL, Tk_Offset(Axis, loose), ALL_MASK},
+	DEF_AXIS_LOOSE, Tk_Offset(Axis, loose),
+	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_BOOLEAN, "-mapped", "x2Mapped", "AxisMapped",
 	DEF_AXIS_ALT_MAPPED, Tk_Offset(Axis, mapped),
 	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_CUSTOM, "-max", "x2Max", "AxisMax",
-	(char *)NULL, 0, ALL_MASK | TK_CONFIG_NULL_OK, &MaxLimitOption},
+	DEF_AXIS_MAX, 0, ALL_MASK | TK_CONFIG_NULL_OK, &MaxLimitOption},
     {TK_CONFIG_CUSTOM, "-min", "x2Min", "AxisMin",
-	(char *)NULL, 0, ALL_MASK | TK_CONFIG_NULL_OK, &MinLimitOption},
+	DEF_AXIS_MIN, 0, ALL_MASK | TK_CONFIG_NULL_OK, &MinLimitOption},
     {TK_CONFIG_DOUBLE, "-rotate", "x2Rotate", "AxisRotate",
-	(char *)NULL, Tk_Offset(Axis, theta), ALL_MASK},
+	DEF_AXIS_ROTATE, Tk_Offset(Axis, theta),
+	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_BOOLEAN, "-showticks", "x2Showticks", "AxisShowticks",
 	DEF_AXIS_TICKS, Tk_Offset(Axis, showTicks), ALL_MASK},
     {TK_CONFIG_DOUBLE, "-stepsize", "x2Stepsize", "AxisStepsize",
-	(char *)NULL, Tk_Offset(Axis, reqStep), ALL_MASK},
+	DEF_AXIS_STEPSIZE, Tk_Offset(Axis, reqStep),
+	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_INT, "-subticks", "x2Subticks", "AxisSubticks",
-	DEF_AXIS_SUBTICKS_GRAPH, Tk_Offset(Axis, reqSubTicks), ALL_MASK},
+	DEF_AXIS_SUBTICKS, Tk_Offset(Axis, reqSubTicks),
+	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_STRING, "-title", "x2Title", "AxisTitle",
-	(char *)NULL, Tk_Offset(Axis, title), ALL_MASK | TK_CONFIG_NULL_OK},
+	DEF_AXIS_TITLE, Tk_Offset(Axis, title), ALL_MASK | TK_CONFIG_NULL_OK},
     {TK_CONFIG_END, NULL, NULL, NULL, NULL, 0, 0}
 };
 
@@ -340,26 +360,32 @@ static Tk_ConfigSpec yAxisConfigSpecs[] =
     {TK_CONFIG_FONT, "-font", "yFont", "AxisFont",
 	DEF_AXIS_FONT, Tk_Offset(Axis, fontPtr), ALL_MASK},
     {TK_CONFIG_BOOLEAN, "-loose", "yLoose", "AxisLoose",
-	(char *)NULL, Tk_Offset(Axis, loose), ALL_MASK},
+	DEF_AXIS_LOOSE, Tk_Offset(Axis, loose),
+	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_PIXELS, "-linewidth", "yLinewidth", "AxisLinewidth",
-	(char *)NULL, Tk_Offset(Axis, lineWidth), ALL_MASK},
+	DEF_AXIS_LINE_WIDTH, Tk_Offset(Axis, lineWidth),
+	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_BOOLEAN, "-logscale", "yLogscale", "AxisLogscale",
-	(char *)NULL, Tk_Offset(Axis, logScale), ALL_MASK},
+	DEF_AXIS_LOG_SCALE, Tk_Offset(Axis, logScale),
+	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_BOOLEAN, "-mapped", "yMapped", "AxisMapped",
 	DEF_AXIS_STD_MAPPED, Tk_Offset(Axis, mapped),
 	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_CUSTOM, "-max", "yMax", "AxisMax",
-	(char *)NULL, 0, ALL_MASK | TK_CONFIG_NULL_OK, &MaxLimitOption},
+	DEF_AXIS_MAX, 0, ALL_MASK | TK_CONFIG_NULL_OK, &MaxLimitOption},
     {TK_CONFIG_CUSTOM, "-min", "yMin", "AxisMin",
-	(char *)NULL, 0, ALL_MASK | TK_CONFIG_NULL_OK, &MinLimitOption},
+	DEF_AXIS_MIN, 0, ALL_MASK | TK_CONFIG_NULL_OK, &MinLimitOption},
     {TK_CONFIG_DOUBLE, "-rotate", "yRotate", "AxisRotate",
-	(char *)NULL, Tk_Offset(Axis, theta), ALL_MASK},
+	DEF_AXIS_ROTATE, Tk_Offset(Axis, theta),
+	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_BOOLEAN, "-showticks", "yShowticks", "AxisShowticks",
-	DEF_AXIS_TICKS, Tk_Offset(Axis, showTicks), ALL_MASK},
+	DEF_AXIS_TICKS, Tk_Offset(Axis, showTicks),
+	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_DOUBLE, "-stepsize", "yStepsize", "AxisStepsize",
-	(char *)NULL, Tk_Offset(Axis, reqStep), ALL_MASK},
+	DEF_AXIS_STEPSIZE, Tk_Offset(Axis, reqStep),
+	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_INT, "-subticks", "ySubticks", "AxisSubticks",
-	DEF_AXIS_SUBTICKS_GRAPH, Tk_Offset(Axis, reqSubTicks), ALL_MASK},
+	DEF_AXIS_SUBTICKS, Tk_Offset(Axis, reqSubTicks), ALL_MASK},
     {TK_CONFIG_STRING, "-title", "yTitle", "AxisTitle",
 	"Y", Tk_Offset(Axis, title), ALL_MASK | TK_CONFIG_NULL_OK},
     {TK_CONFIG_END, NULL, NULL, NULL, NULL, 0, 0}
@@ -374,36 +400,42 @@ static Tk_ConfigSpec y2AxisConfigSpecs[] =
 	DEF_AXIS_FG_MONO, Tk_Offset(Axis, fgColorPtr),
 	TK_CONFIG_MONO_ONLY | ALL_MASK},
     {TK_CONFIG_STRING, "-command", "y2Command", "AxisCommand",
-	(char *)NULL, Tk_Offset(Axis, formatCmd),
+	DEF_AXIS_COMMAND, Tk_Offset(Axis, formatCmd),
 	ALL_MASK | TK_CONFIG_NULL_OK},
     {TK_CONFIG_BOOLEAN, "-descending", "y2Descending", "AxisDescending",
 	DEF_AXIS_DESCENDING, Tk_Offset(Axis, descending),
 	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_FONT, "-font", "y2Font", "AxisFont",
 	DEF_AXIS_FONT, Tk_Offset(Axis, fontPtr), ALL_MASK},
-    {TK_CONFIG_BOOLEAN, "-loose", "y2Loose", "AxisLoose",
-	(char *)NULL, Tk_Offset(Axis, loose), ALL_MASK},
     {TK_CONFIG_PIXELS, "-linewidth", "y2Linewidth", "AxisLinewidth",
-	(char *)NULL, Tk_Offset(Axis, lineWidth), ALL_MASK},
+	DEF_AXIS_LINE_WIDTH, Tk_Offset(Axis, lineWidth),
+	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
+    {TK_CONFIG_BOOLEAN, "-loose", "y2Loose", "AxisLoose",
+	DEF_AXIS_LOOSE, Tk_Offset(Axis, loose),
+	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_BOOLEAN, "-logscale", "y2Logscale", "AxisLogscale",
-	(char *)NULL, Tk_Offset(Axis, logScale), ALL_MASK},
+	DEF_AXIS_LOG_SCALE, Tk_Offset(Axis, logScale),
+	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_BOOLEAN, "-mapped", "y2Mapped", "AxisMapped",
 	DEF_AXIS_ALT_MAPPED, Tk_Offset(Axis, mapped),
 	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_CUSTOM, "-max", "y2Max", "AxisMax",
-	(char *)NULL, 0, ALL_MASK | TK_CONFIG_NULL_OK, &MaxLimitOption},
+	DEF_AXIS_MAX, 0, ALL_MASK | TK_CONFIG_NULL_OK, &MaxLimitOption},
     {TK_CONFIG_CUSTOM, "-min", "y2Min", "AxisMin",
-	(char *)NULL, 0, ALL_MASK | TK_CONFIG_NULL_OK, &MinLimitOption},
+	DEF_AXIS_MIN, 0, ALL_MASK | TK_CONFIG_NULL_OK, &MinLimitOption},
     {TK_CONFIG_DOUBLE, "-rotate", "y2Rotate", "AxisRotate",
-	(char *)NULL, Tk_Offset(Axis, theta), ALL_MASK},
+	DEF_AXIS_ROTATE, Tk_Offset(Axis, theta),
+	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_BOOLEAN, "-showticks", "y2Showticks", "AxisShowticks",
-	DEF_AXIS_TICKS, Tk_Offset(Axis, showTicks), ALL_MASK},
+	DEF_AXIS_TICKS, Tk_Offset(Axis, showTicks),
+	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_DOUBLE, "-stepsize", "y2Stepsize", "AxisStepsize",
-	(char *)NULL, Tk_Offset(Axis, reqStep), ALL_MASK},
+	DEF_AXIS_STEPSIZE, Tk_Offset(Axis, reqStep),
+	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_INT, "-subticks", "y2Subticks", "AxisSubticks",
-	DEF_AXIS_SUBTICKS_GRAPH, Tk_Offset(Axis, reqSubTicks), ALL_MASK},
+	DEF_AXIS_SUBTICKS, Tk_Offset(Axis, reqSubTicks), ALL_MASK},
     {TK_CONFIG_STRING, "-title", "y2Title", "AxisTitle",
-	(char *)NULL, Tk_Offset(Axis, title), ALL_MASK | TK_CONFIG_NULL_OK},
+	DEF_AXIS_TITLE, Tk_Offset(Axis, title), ALL_MASK | TK_CONFIG_NULL_OK},
     {TK_CONFIG_END, NULL, NULL, NULL, NULL, 0, 0}
 };
 
@@ -531,7 +563,7 @@ CreateLabel(graphPtr, axisPtr, value)
     char *result;
 
     if (axisPtr->logScale) {
-	sprintf(string, "1E%d", ROUND(value));
+	sprintf(string, "1E%d", BLT_RND(value));
     } else {
 	if (axisPtr->formatCmd == NULL) {
 	    sprintf(string, "%.*g", NUMDIGITS, value);
@@ -639,7 +671,7 @@ Blt_InvTransform(axis, coord)
     }
     value = (norm * axisPtr->range) + axisPtr->min;
     if (axisPtr->logScale) {
-	value = EXP10(value);
+	value = BLT_EXP10(value);
     }
     return (value);
 }
@@ -758,7 +790,7 @@ Blt_TransformDist(axis, value)
     coord = HORIZONTAL_AXIS(axisPtr)
 	? MAPX(axisPtr, norm) : MAPY(axisPtr, norm);
     coord -= zero;
-    return (ABS(coord));
+    return (BLT_ABS(coord));
 }
 
 /*
@@ -784,9 +816,9 @@ Blt_PointOnGraph(graphPtr, pointPtr)
     Axis *axisPtr;
 
     /*
-     * It doesn't make a difference which x-axis or y-axis we use 
-     * (the point is referenced by window coordinates). We're just 
-     * checking if the point's within the range of axis' normalized 
+     * It doesn't make a difference which x-axis or y-axis we use
+     * (the point is referenced by window coordinates). We're just
+     * checking if the point's within the range of axis' normalized
      * coordinates [0..1].
      */
     axisPtr = (Axis *)graphPtr->bottomAxis;
@@ -876,7 +908,7 @@ UpdateLimits(graphPtr, axisPtr)
 	    } else {
 		double x;
 
-		x = FABS(value) * 0.1;
+		x = BLT_FABS(value) * 0.1;
 		min = value - x;
 		max = value + x;
 	    }
@@ -920,7 +952,7 @@ NiceNum(x, round)
     double nf;			/* nice, rounded fraction */
 
     exponX = floor(log10(x));
-    fractX = x / EXP10(exponX);	/* between 1 and 10 */
+    fractX = x / BLT_EXP10(exponX);	/* between 1 and 10 */
     if (round) {
 	if (fractX < 1.5) {
 	    nf = 1.;
@@ -940,7 +972,7 @@ NiceNum(x, round)
     } else {
 	nf = 10.0;
     }
-    return (nf * EXP10(exponX));
+    return (nf * BLT_EXP10(exponX));
 }
 
 /*
@@ -1047,13 +1079,13 @@ LogAxis(axisPtr)
 	min = UFLOOR(min, axisPtr->step);
 	max = UCEIL(max, axisPtr->step);
 	axisPtr->numTicks = (int)((max - min) / axisPtr->step) + 1;
-	axisPtr->subStep = EXP10(floor(log10(axisPtr->step)));
+	axisPtr->subStep = BLT_EXP10(floor(log10(axisPtr->step)));
 
 	if (axisPtr->step == axisPtr->subStep) {
 	    axisPtr->subTicks = 5;
 	    axisPtr->subStep = axisPtr->step * 0.2;
 	} else {
-	    axisPtr->subTicks = ROUND(axisPtr->step / axisPtr->subStep);
+	    axisPtr->subTicks = BLT_RND(axisPtr->step / axisPtr->subStep);
 	}
     } else {
 	if (min == max) {
@@ -1152,7 +1184,7 @@ static void
 LinearAxis(axisPtr)
     Axis *axisPtr;
 {
-    double range, pad;
+    double range, unit, pad;
     double min, max;
 
     min = axisPtr->limits[LMIN];
@@ -1177,7 +1209,8 @@ LinearAxis(axisPtr)
     axisPtr->tickMin = UFLOOR(min, axisPtr->step) + 0.0;
     axisPtr->tickMax = UCEIL(max, axisPtr->step) + 0.0;
     range = axisPtr->tickMax - axisPtr->tickMin;
-    axisPtr->numTicks = ROUND(range / axisPtr->step) + 1;
+    unit = range / axisPtr->step;
+    axisPtr->numTicks = BLT_RND(unit) + 1;
 
     /*
      * If the axis is "loose", the range is between the two outermost
@@ -1265,7 +1298,8 @@ SetAxisLimits(graphPtr, axisPtr)
 	axisPtr->flags &= ~AXIS_CONFIG_DIRTY;
 
 	/* When any axis changes, we need to layout the entire graph. */
-	graphPtr->flags |= (LAYOUT_PENDING | LAYOUT_ALL | REDRAW_ALL);
+	graphPtr->flags |=
+	    (LAYOUT_NEEDED | LAYOUT_ALL | DIRTY | REFRESH);
     }
 }
 
@@ -1290,7 +1324,7 @@ Blt_ComputeAxes(graphPtr)
 	axisPtr = (Axis *)graphPtr->axisArr[i];
 	SetAxisLimits(graphPtr, axisPtr);
     }
-    graphPtr->flags |= LAYOUT_PENDING;
+    graphPtr->flags |= LAYOUT_NEEDED;
 }
 
 /*
@@ -1397,9 +1431,14 @@ ConfigureAxis(graphPtr, axisPtr, argc, argv, flags)
     }
     axisPtr->lineGC = newGC;
 
+    /*
+     * Don't bother to check what options have changed.  Almost every
+     * axis configuration option changes the size of the plotting area
+     * (except for -foreground).
+     */
     axisPtr->flags |= AXIS_CONFIG_DIRTY;
     SetAxisLimits(graphPtr, axisPtr);
-    Blt_EventuallyRedraw(graphPtr);
+    Blt_RedrawGraph(graphPtr);
     return TCL_OK;
 }
 
@@ -1471,12 +1510,12 @@ CalculateOffsets(graphPtr, axisPtr)
 
     /* Adjust offset for the interior border width and the line width */
     halfLW = axisPtr->lineWidth / 2;
-    pad = graphPtr->intBWidth + halfLW + 1;
-    if (graphPtr->intBWidth > 0) {
+    pad = graphPtr->intBW + halfLW + 1;
+    if (graphPtr->intBW > 0) {
 	pad++;
     }
     textHeight = TEXTHEIGHT(graphPtr->fontPtr);
-    titleOffset = graphPtr->extBWidth + textHeight;
+    titleOffset = graphPtr->extBW + textHeight;
 
     /*
      * Pre-calculate the x-coordinate positions of the baseline,
@@ -1494,7 +1533,7 @@ CalculateOffsets(graphPtr, axisPtr)
 	axisPtr->minorTick = MAPY(oppPtr, -MINOR_TICK) + pad + halfLW;
 	axisPtr->baseline = graphPtr->origin.y + pad;
 	axisPtr->tickLabel = MAPY(oppPtr, -LABEL_TICK) +
-	    (axisPtr->lineWidth + graphPtr->intBWidth + pad);
+	    (axisPtr->lineWidth + graphPtr->intBW + pad);
 	axisPtr->titleX = (graphPtr->extreme.x + graphPtr->origin.x) / 2;
 	axisPtr->titleY = graphPtr->height - titleOffset;
 	axisPtr->anchor = TK_ANCHOR_N;
@@ -1505,7 +1544,7 @@ CalculateOffsets(graphPtr, axisPtr)
 	axisPtr->minorTick = MAPX(oppPtr, -MINOR_TICK) - (pad + halfLW);
 	axisPtr->baseline = graphPtr->origin.x - pad;
 	axisPtr->tickLabel = MAPX(oppPtr, -LABEL_TICK) -
-	    (axisPtr->lineWidth + graphPtr->intBWidth + pad);
+	    (axisPtr->lineWidth + graphPtr->intBW + pad);
 	axisPtr->titleX = titleOffset;
 	axisPtr->titleY = (graphPtr->origin.y + graphPtr->extreme.y) / 2;
 	axisPtr->anchor = TK_ANCHOR_E;
@@ -1518,7 +1557,7 @@ CalculateOffsets(graphPtr, axisPtr)
 	    MAPY(oppPtr, 1.0 + MINOR_TICK) - (pad + halfLW);
 	axisPtr->baseline = graphPtr->extreme.y - pad;
 	axisPtr->tickLabel = MAPY(oppPtr, 1.0 + LABEL_TICK) -
-	    (axisPtr->lineWidth + graphPtr->intBWidth + pad);
+	    (axisPtr->lineWidth + graphPtr->intBW + pad);
 	axisPtr->titleX = (graphPtr->extreme.x + graphPtr->origin.x) / 2;
 	x2Offset = (graphPtr->title != NULL) ? 2 * textHeight : textHeight / 2;
 	axisPtr->titleY = titleOffset + x2Offset;
@@ -1530,7 +1569,7 @@ CalculateOffsets(graphPtr, axisPtr)
 	axisPtr->minorTick = MAPX(oppPtr, 1.0 + MINOR_TICK) + pad + halfLW;
 	axisPtr->baseline = graphPtr->extreme.x + pad;
 	axisPtr->tickLabel = MAPX(oppPtr, 1.0 + LABEL_TICK) +
-	    (axisPtr->lineWidth + graphPtr->intBWidth + pad);
+	    (axisPtr->lineWidth + graphPtr->intBW + pad);
 	axisPtr->titleX = graphPtr->width -
 	    (graphPtr->legendPtr->width + titleOffset);
 	axisPtr->titleY = (graphPtr->origin.y + graphPtr->extreme.y) / 2;
@@ -1632,6 +1671,7 @@ LayoutAxis(graphPtr, axis)
     double value, subValue;
     register int i, j;
     register int sgmts, labels;
+    double epsilon;
     static float logTable[] =	/* Precomputed log10 values [1..10] */
     {
 	0.0, 0.301, 0.477, 0.602, 0.699, 0.778, 0.845, 0.903, 0.954, 1.0
@@ -1662,8 +1702,8 @@ LayoutAxis(graphPtr, axis)
 	goto done;		/* Only display axis line */
     }
     /* Use numbers just beyond the limits when testing for equality */
-    min -= (axisPtr->subStep * 0.001);
-    max += (axisPtr->subStep * 0.001);
+    epsilon = (axisPtr->subStep * 0.1);
+    min -= epsilon, max += epsilon;
 
     value = axisPtr->tickMin;	/* Start from smallest axis tick */
     for (i = 0; i < axisPtr->numTicks; i++) {
@@ -1725,7 +1765,7 @@ LayoutAxis(graphPtr, axis)
  *	Draws the axis, ticks, and labels onto the canvas.
  *
  *	Initializes and passes text attribute information through
- *	TextAttr structure.
+ *	TextAttributes structure.
  *
  * Results:
  *	None.
@@ -1745,7 +1785,7 @@ static void
 DisplayAxis(graphPtr, axis, attrPtr)
     Graph *graphPtr;
     GraphAxis *axis;
-    TextAttr *attrPtr;
+    TextAttributes *attrPtr;
 {
     Axis *axisPtr = (Axis *)axis;
 
@@ -1756,7 +1796,7 @@ DisplayAxis(graphPtr, axis, attrPtr)
     }
     if (axisPtr->numTicks > 0) {
 	register int i;
-	TextAttr textAttr;
+	TextAttributes textAttr;
 
 	/* Setup static text attribute information */
 
@@ -1789,7 +1829,7 @@ DisplayAxis(graphPtr, axis, attrPtr)
  *	labels.
  *
  *	Initializes and passes text attribute information through
- *	TextAttr structure.
+ *	TextAttributes structure.
  *
  * Results:
  *	None.
@@ -1803,7 +1843,7 @@ static void
 PrintAxis(graphPtr, axis, attrPtr)
     Graph *graphPtr;
     GraphAxis *axis;
-    TextAttr *attrPtr;
+    TextAttributes *attrPtr;
 {
     Axis *axisPtr = (Axis *)axis;
 
@@ -1813,7 +1853,7 @@ PrintAxis(graphPtr, axis, attrPtr)
 	    axisPtr->titleX, axisPtr->titleY);
     }
     if (axisPtr->numTicks > 0) {
-	TextAttr textAttr;
+	TextAttributes textAttr;
 	register int i;
 
 	/* Setup static text attribute information */
@@ -2006,12 +2046,15 @@ Blt_ComputeLayout(graphPtr)
 {
     int left, right, top, bottom;
     int maxTickWidth;
-    int leftOver, borderWidths;
-    int lineHeight = TEXTHEIGHT(graphPtr->fontPtr);
+    int height;
+    int leftOver;
+    unsigned int borderWidths;
+    unsigned int lineHeight = TEXTHEIGHT(graphPtr->fontPtr);
     Axis *x1, *x2, *y1, *y2;
-    int twiceHeight = (2 * lineHeight);
-    int halfHeight = (lineHeight / 2);
+    unsigned int twiceHeight = (2 * lineHeight);
+    unsigned int halfHeight = (lineHeight / 2);
     double normYScale, normXScale;
+    double value;
 
     x1 = (Axis *)graphPtr->bottomAxis;
     x2 = (Axis *)graphPtr->topAxis;
@@ -2045,16 +2088,6 @@ Blt_ComputeLayout(graphPtr)
 	right += y2->width + PADX;
 	normXScale += LABEL_TICK;
     }
-    (*graphPtr->legendPtr->geomProc) (graphPtr);
-    if ((graphPtr->legendPtr->mapped) && (graphPtr->legendPtr->useDefault)) {
-	right += graphPtr->legendPtr->width;
-    } else {
-	right += halfHeight;
-    }
-    maxTickWidth = MAX(x1->width, x2->width) / 2;
-    if (right < maxTickWidth) {
-	right = maxTickWidth;
-    }
     /* Override calculated values if user specified margins */
 
     if (graphPtr->leftMargin > 0) {
@@ -2066,10 +2099,21 @@ Blt_ComputeLayout(graphPtr)
     if (graphPtr->bottomMargin > 0) {
 	bottom = graphPtr->bottomMargin;
     }
+    borderWidths = graphPtr->extBW + graphPtr->intBW;
+    height = graphPtr->height - ((2 * borderWidths) + top + bottom);
+    (*graphPtr->legendPtr->geomProc) (graphPtr, height);
+    if ((graphPtr->legendPtr->mapped) && (graphPtr->legendPtr->useDefault)) {
+	right += graphPtr->legendPtr->width;
+    } else {
+	right += halfHeight;
+    }
+    maxTickWidth = BLT_MAX(x1->width, x2->width) / 2;
+    if (right < maxTickWidth) {
+	right = maxTickWidth;
+    }
     if (graphPtr->rightMargin > 0) {
 	right = graphPtr->rightMargin;
     }
-    borderWidths = graphPtr->extBWidth + graphPtr->intBWidth;
     top += borderWidths;
     left += borderWidths;
     right += borderWidths;
@@ -2092,8 +2136,10 @@ Blt_ComputeLayout(graphPtr)
 
     /* Add tick distance to center graph */
 
-    x1->offset += ROUND(LABEL_TICK * x1->scale);
-    y1->offset -= ROUND(LABEL_TICK * y1->scale);
+    value = LABEL_TICK * x1->scale;
+    x1->offset += BLT_RND(value);
+    value = LABEL_TICK * y1->scale;
+    y1->offset -= BLT_RND(value);
 
     x2->scale = x1->scale;
     x2->offset = x1->offset;
@@ -2102,7 +2148,8 @@ Blt_ComputeLayout(graphPtr)
 
     /* Calculate the average symbol (formula is arbitrary) */
 
-    graphPtr->avgSymSize = ROUND(log(((double)x1->scale) * y1->scale) * 0.8);
+    value = log(((double)x1->scale) * y1->scale) * 0.8;
+    graphPtr->avgSymSize = BLT_RND(value);
 
     graphPtr->origin.x = MAPX(x1, 0.0);
     graphPtr->origin.y = MAPY(y1, 0.0);
@@ -2256,9 +2303,8 @@ Blt_CreateAxis(graphPtr, type, flags)
     axisPtr->type = type;
     axisPtr->location = location;
     axisPtr->interp = graphPtr->interp;	/* Needed for Tcl_PrintDouble */
-    axisPtr->flags = 0;
-    axisPtr->step = 1.0;
-    axisPtr->theta = 0.0;
+    axisPtr->showTicks = 1;
+    axisPtr->reqSubTicks = 2;
     axisPtr->destroyProc = DestroyAxis;
     axisPtr->displayProc = DisplayAxis;
     axisPtr->layoutProc = LayoutAxis;
