@@ -34,14 +34,75 @@ extern int strcasecmp _ANSI_ARGS_((CONST char *s1, CONST char *s2));
 extern char *strdup _ANSI_ARGS_((CONST char *s));
 #endif
 
+static int ParsePosition _ANSI_ARGS_((ClientData, Tcl_Interp *, Tk_Window,
+	char *, char *, int));
+static char *PrintPosition _ANSI_ARGS_((ClientData, Tk_Window, char *, int,
+	Tcl_FreeProc **));
+
+Tk_CustomOption bltPositionOption =
+{
+    ParsePosition, PrintPosition, (ClientData)0
+};
+
+static int ParseAxisFlags _ANSI_ARGS_((ClientData, Tcl_Interp *, Tk_Window,
+	char *, char *, int));
+static char *PrintAxisFlags _ANSI_ARGS_((ClientData, Tk_Window, char *, int,
+	Tcl_FreeProc **));
+
+Tk_CustomOption bltXAxisFlagsOption =
+{
+    ParseAxisFlags, PrintAxisFlags, (ClientData)ANY_X_MASK
+};
+
+Tk_CustomOption bltYAxisFlagsOption =
+{
+    ParseAxisFlags, PrintAxisFlags, (ClientData)ANY_Y_MASK
+};
+
+
 /* ----------------------------------------------------------------------
  * Custom option parse and print procedures
  * ----------------------------------------------------------------------
  */
+
+int
+Blt_GetPosition(interp, string, pointPtr)
+    Tcl_Interp *interp;
+    char *string;
+    XPoint *pointPtr;
+{
+    char *strX, *strY;
+    int x, y;
+
+    if ((string == NULL) || (*string == '\0')) {
+	pointPtr->y = pointPtr->x = DEF_POSITION;
+	return TCL_OK;		/* Position marked as default */
+    }
+    if (*string != '@') {
+	Tcl_AppendResult(interp, "bad position \"", string,
+	    "\": must start with an '@'", (char *)NULL);
+	return TCL_ERROR;
+    }
+    strX = string + 1;
+    strY = strchr(strX, ',');
+    if (strY == NULL) {
+	Tcl_AppendResult(interp, "bad position \"", string,
+	    "\": should be \"@x,y\"", (char *)NULL);
+	return TCL_ERROR;
+    }
+    *strY++ = '\0';
+    if ((Tcl_GetInt(interp, strX, &x) != TCL_OK) ||
+	(Tcl_GetInt(interp, strY, &y) != TCL_OK)) {
+	return TCL_ERROR;
+    }
+    pointPtr->x = x, pointPtr->y = y;
+    return TCL_OK;
+}
+
 /*
  *----------------------------------------------------------------------
  *
- * ParseCoords --
+ * ParsePosition --
  *
  *	Convert the string representation of a legend XY position into
  *	window coordinates.  The form of the string must be "@x,y" or
@@ -58,8 +119,8 @@ extern char *strdup _ANSI_ARGS_((CONST char *s));
  *----------------------------------------------------------------------
  */
 /*ARGSUSED*/
-int
-Blt_ParseCoords(clientData, interp, tkwin, value, widgRec, offset)
+static int
+ParsePosition(clientData, interp, tkwin, value, widgRec, offset)
     ClientData clientData;	/* not used */
     Tcl_Interp *interp;		/* Interpreter to send results back to */
     Tk_Window tkwin;		/* not used */
@@ -68,38 +129,13 @@ Blt_ParseCoords(clientData, interp, tkwin, value, widgRec, offset)
     int offset;			/* offset to XPoint structure */
 {
     XPoint *pointPtr = (XPoint *)(widgRec + offset);
-    char *stringX, *stringY;
-    int x, y;
-
-    if ((value == NULL) || (*value == '\0')) {
-	pointPtr->y = pointPtr->x = DEF_POSITION;
-	return TCL_OK;		/* Position marked as default */
-    }
-    if (*value != '@') {
-	Tcl_AppendResult(interp, "bad position \"", value,
-	    "\": must start with an '@'", (char *)NULL);
-	return TCL_ERROR;
-    }
-    stringX = value + 1;
-    stringY = strchr(stringX, ',');
-    if (stringY == NULL) {
-	Tcl_AppendResult(interp, "bad position \"", value,
-	    "\": should be \"@x,y\"", (char *)NULL);
-	return TCL_ERROR;
-    }
-    *stringY++ = '\0';
-    if ((Tcl_GetInt(interp, stringX, &x) != TCL_OK) ||
-	(Tcl_GetInt(interp, stringY, &y) != TCL_OK)) {
-	return TCL_ERROR;
-    }
-    pointPtr->x = x, pointPtr->y = y;
-    return TCL_OK;
+    return (Blt_GetPosition(interp, value, pointPtr));
 }
 
 /*
  *----------------------------------------------------------------------
  *
- * Blt_PrintCoords --
+ * PrintPosition --
  *
  *	Convert the window coordinates into a string.
  *
@@ -109,8 +145,8 @@ Blt_ParseCoords(clientData, interp, tkwin, value, widgRec, offset)
  *----------------------------------------------------------------------
  */
 /*ARGSUSED*/
-char *
-Blt_PrintCoords(clientData, tkwin, widgRec, offset, freeProcPtr)
+static char *
+PrintPosition(clientData, tkwin, widgRec, offset, freeProcPtr)
     ClientData clientData;	/* not used */
     Tk_Window tkwin;		/* not used */
     char *widgRec;		/* Widget record */
@@ -148,8 +184,8 @@ Blt_PrintCoords(clientData, tkwin, widgRec, offset, freeProcPtr)
  *----------------------------------------------------------------------
  */
 /*ARGSUSED*/
-int
-Blt_ParseAxisFlags(clientData, interp, tkwin, value, widgRec, offset)
+static int
+ParseAxisFlags(clientData, interp, tkwin, value, widgRec, offset)
     ClientData clientData;	/* not used */
     Tcl_Interp *interp;		/* Interpreter to send results back to */
     Tk_Window tkwin;		/* not used */
@@ -199,7 +235,7 @@ Blt_ParseAxisFlags(clientData, interp, tkwin, value, widgRec, offset)
 /*
  *----------------------------------------------------------------------
  *
- * Blt_PrintAxisFlags --
+ * PrintAxisFlags --
  *
  *	Convert the window coordinates into a string.
  *
@@ -209,8 +245,8 @@ Blt_ParseAxisFlags(clientData, interp, tkwin, value, widgRec, offset)
  *----------------------------------------------------------------------
  */
 /*ARGSUSED*/
-char *
-Blt_PrintAxisFlags(clientData, tkwin, widgRec, offset, freeProcPtr)
+static char *
+PrintAxisFlags(clientData, tkwin, widgRec, offset, freeProcPtr)
     ClientData clientData;	/* not used */
     Tk_Window tkwin;		/* not used */
     char *widgRec;		/* Widget record */
@@ -410,8 +446,8 @@ Blt_GetBoundingBox(width, height, theta, rotWPtr, rotHPtr, pointArr)
 	    maxY = y;
 	}
 	if (pointArr != NULL) {
-	    pointArr[i].x = ROUND(x);
-	    pointArr[i].y = ROUND(y);
+	    pointArr[i].x = BLT_RND(x);
+	    pointArr[i].y = BLT_RND(y);
 	}
     }
 
@@ -576,9 +612,11 @@ Blt_RotateBitmap(display, draw, bitmapGC, srcBitmap, srcWidth, srcHeight,
 		rotY = (transX * sinTheta) + (transY * cosTheta);
 
 		/* Translate back to the center of the source image */
+		rotX += srcX;
+		rotY += srcY;
 
-		sx = ROUND(srcX + rotX);
-		sy = ROUND(srcY + rotY);
+		sx = BLT_RND(rotX);
+		sy = BLT_RND(rotY);
 
 		/*
 		 * Verify the coordinates, since the destination image can be
@@ -743,7 +781,7 @@ Blt_DrawText(display, draw, text, attrPtr, x, y)
     Display *display;
     Drawable draw;
     char *text;
-    TextAttr *attrPtr;		/* Text attribute information */
+    TextAttributes *attrPtr;	/* Text attribute information */
     int x;			/* Window x coordinate */
     int y;			/* Window y coordinate */
 {
@@ -780,3 +818,46 @@ Blt_DrawText(display, draw, text, attrPtr, x, y)
 	XFreePixmap(display, bitmap);
     }
 }
+
+/*
+ * -----------------------------------------------------------------
+ *
+ * Blt_FindToken --
+ *
+ *	Linearly search through a list of strings.  Returns the
+ *      the index of *key*.
+ *
+ * Results:
+ *	Returns index of *key* if found, -1 otherwise.
+ *
+ * -----------------------------------------------------------------
+ */
+
+int
+Blt_GetTokenIndex(tokenList, key, ignoreCase)
+    char **tokenList;
+    char *key;
+    int ignoreCase;
+{
+    int length;
+    register int count;
+    register char **p;
+    int found;
+    char c;
+
+    length = strlen(key);
+    count = 0;
+    c = *key;
+    for (p = tokenList; *p != NULL; p++) {
+	if (ignoreCase) {
+	    found = (strncasecmp(key, *p, length) == 0);
+	} else {
+	    found = ((c == **p) && (strncmp(key, *p, length) == 0));
+	}
+	if (found) {
+	    return (count);
+	}
+	count++;
+    }
+    return -1;
+}
