@@ -39,7 +39,6 @@
 #include <X11/Xutil.h>
 #include <X11/Xatom.h>
 
-#define MIN(a,b)        (((a)<(b))?(a):(b))
 #define RED(c)          ((double)(c)->red/65536.0)
 #define GREEN(c)        ((double)(c)->green/65536.0)
 #define BLUE(c)         ((double)(c)->blue/65536.0)
@@ -53,11 +52,10 @@ typedef enum ColorLevels {
     MONOCHROME_LEVEL, GRAYSCALE_LEVEL, FULLCOLOR_LEVEL
 } ColorLevel;
 
-static char *colorModes[] =
+static char *modeTokens[] =
 {
-    "monochrome", "grayscale", "color"
+    "monochrome", "grayscale", "color", (char *)NULL,
 };
-static int numColorModes = sizeof(colorModes) / sizeof(char *);
 
 /*
  * Sun's bundled and unbundled C compilers choke on static function
@@ -77,7 +75,7 @@ static char *PrintColorMode _ANSI_ARGS_((ClientData clientData,
 
 static Tk_CustomOption ColorModeOption =
 {
-    ParseColorMode, PrintColorMode, (ClientData)NULL,
+    ParseColorMode, PrintColorMode, (ClientData)0,
 };
 
 /*
@@ -96,7 +94,7 @@ typedef struct {
     PostScriptDestroyProc *destroyProc;
 
     int pageWidth, pageHeight;	/* Requested page extents */
-    int pageX, pageY;		/* Requested page anchor */
+    int xOffset, yOffset;	/* Requested page anchor */
     Tk_Anchor anchor;		/* Anchor type for positioning page */
     ColorLevel colorLevel;	/* If zero, use BW color scheme */
     char *colorVarName;		/* If non-NULL, name of color map variable
@@ -112,27 +110,43 @@ typedef struct {
 				 * to printer points (pica) */
 } PostScript;
 
+#define DEF_PS_COLOR_MAP	(char *)NULL
+#define DEF_PS_COLOR_MODE	"color"
+#define DEF_PS_FONT_MAP		(char *)NULL
+#define DEF_PS_LANDSCAPE	"0"
+#define DEF_PS_PAGE_ANCHOR	"nw"
+#define DEF_PS_PAGE_HEIGHT	"0"
+#define DEF_PS_PAGE_WIDTH	"0"
+#define DEF_PS_PAGE_X_OFFSET	"1i"
+#define DEF_PS_PAGE_Y_OFFSET	"1i"
 
 static Tk_ConfigSpec configSpecs[] =
 {
     {TK_CONFIG_CUSTOM, "-colormode", "colorMode", (char *)NULL,
-	(char *)NULL, Tk_Offset(PostScript, colorLevel), 0, &ColorModeOption},
+	DEF_PS_COLOR_MODE, Tk_Offset(PostScript, colorLevel),
+	TK_CONFIG_DONT_SET_DEFAULT, &ColorModeOption},
     {TK_CONFIG_STRING, "-colormap", "colorMap", (char *)NULL,
-	(char *)NULL, Tk_Offset(PostScript, colorVarName), 0},
+	DEF_PS_COLOR_MAP, Tk_Offset(PostScript, colorVarName), 0},
     {TK_CONFIG_STRING, "-fontmap", "fontMap", (char *)NULL,
-	(char *)NULL, Tk_Offset(PostScript, fontVarName), 0},
+	DEF_PS_FONT_MAP, Tk_Offset(PostScript, fontVarName), 0},
     {TK_CONFIG_BOOLEAN, "-landscape", "landscape", (char *)NULL,
-	(char *)NULL, Tk_Offset(PostScript, landscape), 0},
+	DEF_PS_LANDSCAPE, Tk_Offset(PostScript, landscape),
+	TK_CONFIG_DONT_SET_DEFAULT},
     {TK_CONFIG_ANCHOR, "-pageanchor", "pageAnchor", (char *)NULL,
-	(char *)NULL, Tk_Offset(PostScript, anchor), 0},
+	DEF_PS_PAGE_ANCHOR, Tk_Offset(PostScript, anchor),
+	TK_CONFIG_DONT_SET_DEFAULT},
     {TK_CONFIG_PIXELS, "-pageheight", "pageHeight", (char *)NULL,
-	(char *)NULL, Tk_Offset(PostScript, pageHeight), 0},
+	DEF_PS_PAGE_HEIGHT, Tk_Offset(PostScript, pageHeight),
+	TK_CONFIG_DONT_SET_DEFAULT},
     {TK_CONFIG_PIXELS, "-pagewidth", "pageWidth", (char *)NULL,
-	(char *)NULL, Tk_Offset(PostScript, pageWidth), 0},
+	DEF_PS_PAGE_WIDTH, Tk_Offset(PostScript, pageWidth),
+	TK_CONFIG_DONT_SET_DEFAULT},
     {TK_CONFIG_PIXELS, "-pagex", "pageX", (char *)NULL,
-	(char *)NULL, Tk_Offset(PostScript, pageX), 0},
+	DEF_PS_PAGE_X_OFFSET, Tk_Offset(PostScript, xOffset),
+	TK_CONFIG_DONT_SET_DEFAULT},
     {TK_CONFIG_PIXELS, "-pagey", "pageY", (char *)NULL,
-	(char *)NULL, Tk_Offset(PostScript, pageY), 0},
+	DEF_PS_PAGE_Y_OFFSET, Tk_Offset(PostScript, yOffset),
+	TK_CONFIG_DONT_SET_DEFAULT},
     {TK_CONFIG_END, NULL, NULL, NULL, NULL, 0, 0}
 };
 
@@ -177,26 +191,21 @@ ParseColorMode(clientData, interp, tkwin, value, widgRec, offset)
     int offset;			/* Offset of colorLevel field in record */
 {
     ColorLevel *colorLevel = (ColorLevel *)(widgRec + offset);
-    register int i;
-    int length;
-    char c;
+    int index;
 
     if (value == NULL || *value == '\0') {
 	*colorLevel = FULLCOLOR_LEVEL;
 	return TCL_OK;
     }
-    c = *value;
-    length = strlen(value);
-    for (i = 0; i < numColorModes; i++) {
-	if ((c == colorModes[i][0]) &&
-	    (strncmp(value, colorModes[i], length) == 0)) {
-	    *colorLevel = (ColorLevel)i;
-	    return TCL_OK;
-	}
+    index = Blt_GetTokenIndex(modeTokens, value, 0);
+    if (index < 0) {
+	Tcl_AppendResult(interp, "bad color mode \"", value,
+	    "\": should be ", "\"color\", \"grayscale\", or \"monochrome\"",
+	    (char *)NULL);
+	return TCL_ERROR;
     }
-    Tcl_AppendResult(interp, "bad color mode \"", value, "\": should be ",
-	"\"color\", \"grayscale\", or \"monochrome\"", (char *)NULL);
-    return TCL_ERROR;
+    *colorLevel = (ColorLevel)index;
+    return TCL_OK;
 }
 
 /*
@@ -221,9 +230,9 @@ PrintColorMode(clientData, tkwin, widgRec, offset, freeProcPtr)
     int offset;			/* field of colorLevel in record */
     Tcl_FreeProc **freeProcPtr;	/* not used */
 {
-    ColorLevel colorLevel = *(ColorLevel *)(widgRec + offset);
+    ColorLevel level = *(ColorLevel *)(widgRec + offset);
 
-    return (colorModes[(int)colorLevel]);
+    return (modeTokens[level]);
 }
 
 /*
@@ -878,7 +887,7 @@ void
 Blt_TextToPostScript(graphPtr, text, attrPtr, x, y)
     Graph *graphPtr;
     char *text;			/* Text string to print */
-    TextAttr *attrPtr;		/* Text attribute information: rotation,
+    TextAttributes *attrPtr;	/* Text attribute information: rotation,
 				 * font, background color, anchor */
     int x, y;			/* Window coordinates of anchor */
 {
@@ -960,6 +969,9 @@ Blt_PrintLine(graphPtr, pointArr, numPoints)
 	}
 	Tcl_AppendResult(graphPtr->interp, graphPtr->scratchPtr, (char *)NULL);
     }
+    /*
+     * Note: It's assumed that i is numPoints - 1 after finishing loop
+     */
     sprintf(graphPtr->scratchPtr, "%d %d lineto\nDashesProc stroke\n",
 	pointArr[i].x, pointArr[i].y);
     Tcl_AppendResult(graphPtr->interp, graphPtr->scratchPtr, (char *)NULL);
@@ -1202,7 +1214,25 @@ PrintElements(graphPtr)
 	elemPtr = (Element *)Blt_GetListValue(entryPtr);
 	Tcl_AppendResult(graphPtr->interp, "\n% Element \"", elemPtr->id,
 	    "\"\n\n", (char *)NULL);
-	(*elemPtr->printProc) (graphPtr, elemPtr);
+	(*elemPtr->printProc) (graphPtr, elemPtr, ELEM_NORMAL);
+    }
+}
+
+static void
+PrintActiveElements(graphPtr)
+    Graph *graphPtr;
+{
+    Blt_ListEntry *entryPtr;
+    register Element *elemPtr;
+
+    for (entryPtr = Blt_FirstListEntry(&(graphPtr->elemList));
+	entryPtr != NULL; entryPtr = Blt_NextListEntry(entryPtr)) {
+	elemPtr = (Element *)Blt_GetListValue(entryPtr);
+	if (elemPtr->flags & ACTIVE) {
+	    Tcl_AppendResult(graphPtr->interp, "\n% Active Element \"",
+		elemPtr->id, "\"\n\n", (char *)NULL);
+	    (*elemPtr->printProc) (graphPtr, elemPtr, ELEM_ACTIVE);
+	}
     }
 }
 
@@ -1215,7 +1245,7 @@ PrintExterior(graphPtr, regionPtr)
     register int i;
     int x, y;
     XRectangle rectArr[4];
-    TextAttr textAttr;
+    TextAttributes textAttr;
     GraphAxis *axisPtr;
 
     rectArr[0].x = rectArr[0].y = rectArr[3].x = rectArr[1].x = 0;
@@ -1236,10 +1266,10 @@ PrintExterior(graphPtr, regionPtr)
 	    rectArr[i].y, rectArr[i].width, rectArr[i].height);
     }
     /* Interior 3D border */
-    if ((graphPtr->intRelief != TK_RELIEF_FLAT) && (graphPtr->intBWidth > 0)) {
+    if ((graphPtr->intRelief != TK_RELIEF_FLAT) && (graphPtr->intBW > 0)) {
 	Blt_Print3DRectangle(graphPtr, graphPtr->border, regionPtr->x,
 	    regionPtr->y, regionPtr->width, regionPtr->height,
-	    graphPtr->intBWidth, graphPtr->intRelief);
+	    graphPtr->intBW, graphPtr->intRelief);
     }
     /* Print legend if using default location */
     if (graphPtr->legendPtr->useDefault) {
@@ -1253,7 +1283,7 @@ PrintExterior(graphPtr, regionPtr)
 
     if (graphPtr->title != NULL) {
 	x = (graphPtr->extreme.x + graphPtr->origin.x) / 2;
-	y = graphPtr->extBWidth + TEXTHEIGHT(graphPtr->fontPtr);
+	y = graphPtr->extBW + TEXTHEIGHT(graphPtr->fontPtr);
 	Blt_TextToPostScript(graphPtr, graphPtr->title, &textAttr, x, y);
     }
     for (i = 0; i < 4; i++) {	/* Print axes */
@@ -1315,8 +1345,8 @@ PrintPostScript(graphPtr, argc, argv)
 	goto error;
     }
     /* Scale coordinates into printer points (pica) */
-    psPtr->x = psPtr->pageX * psPtr->pointScale;
-    psPtr->y = psPtr->pageY * psPtr->pointScale;
+    psPtr->x = psPtr->xOffset * psPtr->pointScale;
+    psPtr->y = psPtr->yOffset * psPtr->pointScale;
     psPtr->width = graphPtr->width;
     if (psPtr->pageWidth > 0) {
 	psPtr->width = psPtr->pageWidth * psPtr->pointScale;
@@ -1342,12 +1372,12 @@ PrintPostScript(graphPtr, argc, argv)
     if (PrintPreamble(graphPtr, fileName) != TCL_OK) {
 	goto error;
     }
-    twiceBW = (graphPtr->intBWidth + graphPtr->intBWidth);
+    twiceBW = (graphPtr->intBW + graphPtr->intBW);
     /*
      * Determine rectangle of the plotting surface the graph window
      */
-    clipRect.x = graphPtr->origin.x - graphPtr->intBWidth;
-    clipRect.y = graphPtr->extreme.y - graphPtr->intBWidth;
+    clipRect.x = graphPtr->origin.x - graphPtr->intBW;
+    clipRect.y = graphPtr->extreme.y - graphPtr->intBW;
     clipRect.width = (graphPtr->extreme.x - graphPtr->origin.x) + twiceBW;
     clipRect.height = (graphPtr->origin.y - graphPtr->extreme.y) + twiceBW;
 
@@ -1359,18 +1389,12 @@ PrintPostScript(graphPtr, argc, argv)
     /*
      * Draw the elements and tags on the interior of the plotting surface
      */
+    PrintElements(graphPtr);
+    PrintTags(graphPtr);
+    PrintActiveElements(graphPtr);
     if (!legendPtr->useDefault) {
 	(*legendPtr->printProc) (graphPtr);
     }
-    /* If this is a graph, draw the elements on top of tags.  On barcharts,
-     * it's the opposite. */
-    if (graphPtr->type == GRAPH) {
-	PrintTags(graphPtr);
-	PrintElements(graphPtr);
-    } else {
-	PrintElements(graphPtr);
-	PrintTags(graphPtr);
-    }
     Tcl_AppendResult(graphPtr->interp, "\n% Unset clipping\ngrestore\n\n",
 	(char *)NULL);
     PrintExterior(graphPtr, &clipRect);
@@ -1395,10 +1419,15 @@ PrintPostScript(graphPtr, argc, argv)
     if (f != NULL) {
 	fclose(f);
     }
-    graphPtr->flags = (LAYOUT_PENDING | LAYOUT_ALL);
     /* Reset height and width of graph window */
     graphPtr->width = Tk_Width(graphPtr->tkwin);
     graphPtr->height = Tk_Height(graphPtr->tkwin);
+    graphPtr->flags = (LAYOUT_NEEDED | LAYOUT_ALL);
+    /*
+     * Redraw the graph in order to re-calculate the layout as soon
+     * as possible. This is in the case the crosshairs are active.
+     */
+    Blt_RedrawGraph(graphPtr);
     return (result);
 }
 
@@ -1442,12 +1471,11 @@ Blt_CreatePostScript(graphPtr)
 #define MM_PER_INCH 25.4
     pixelsPerInch = (MM_PER_INCH * (double)screenWidth / screenWidthMM);
 #define PICA_PER_INCH 72.0
-    psPtr->pointScale = PICA_PER_INCH / pixelsPerInch;
-
     /*
      * Set margins to one inch
      */
-    psPtr->pageX = psPtr->pageY = (int)(pixelsPerInch + 0.5);
+    psPtr->pointScale = PICA_PER_INCH / pixelsPerInch;
+    psPtr->xOffset = psPtr->yOffset = (int)(pixelsPerInch + 0.5);
     graphPtr->postscript = (GraphPostScript *)psPtr;
     return TCL_OK;
 }
