@@ -26,13 +26,14 @@
  * Graph widget created by Sani Nassif and George Howlett.
  */
 
-/* To do:
+/*
+ * To do:
  *
  * 1) Fix log manual scale for log axes.
  *
  * 2) Update manual pages.
  *
- * 3) Update internal comments.
+ * 3) Update comments.
  *
  * 5) Contour and flow graphs
  *
@@ -40,8 +41,6 @@
  *
  * 7) Arrows for line tags
  *
- * 8) Cross and plus symbols look dopey when < 5 pixels (use lines instead)
- *
  * 9) Make sure reasonable defaults show for configuration items
  *
  * 10) Rubberbox for zooming???
@@ -55,40 +54,42 @@
 #include <X11/Xatom.h>
 
 #ifndef GRAPH_VERSION
-#define GRAPH_VERSION "3.3"
+#define GRAPH_VERSION "4.0"
 #endif
 
-#define TRUE  1
-#define FALSE 0
-
 static char *classNames[] =
 {
     "Blt_graph", "Blt_barchart"
 };
+
 static unsigned int configFlags[] =
 {
     XYGRAPH_MASK, BARCHART_MASK
 };
 
-#define DEF_BAR_WIDTH		"0.95"
-#define DEF_GRAPH_HALO		"0.5i"
+#define DEF_GRAPH_BAR_WIDTH	"0.95"
 #define DEF_GRAPH_BG_COLOR	BISQUE1
 #define DEF_GRAPH_BG_MONO	WHITE
+#define DEF_GRAPH_BORDER_WIDTH	"0"
+#define DEF_GRAPH_BORDER_WIDTH  "0"
+#define DEF_GRAPH_BUFFERED	"1"
 #define DEF_GRAPH_CURSOR  	"crosshair"
 #define DEF_GRAPH_FG_COLOR	BLACK
 #define DEF_GRAPH_FG_MONO	BLACK
 #define DEF_GRAPH_FONT		"*-Helvetica-Bold-R-Normal-*-120-*"
+#define DEF_GRAPH_HALO		"0.5i"
+#define DEF_GRAPH_HALO_BAR	"0.1i"
 #define DEF_GRAPH_HEIGHT	"400"
+#define DEF_GRAPH_INVERT_XY	"0"
+#define DEF_GRAPH_MARGIN	"0"
+#define DEF_GRAPH_PLOT_BG_COLOR	ANTIQUEWHITE1
+#define DEF_GRAPH_PLOT_BG_MONO	WHITE
+#define DEF_GRAPH_PLOT_BW_COLOR "2"
+#define DEF_GRAPH_PLOT_BW_MONO  "0"
+#define DEF_GRAPH_PLOT_RELIEF	"sunken"
+#define DEF_GRAPH_RELIEF	"flat"
+#define DEF_GRAPH_TITLE		(char *)NULL
 #define DEF_GRAPH_WIDTH		"400"
-#define DEF_EXT_BORDER_WIDTH   	(char *)NULL
-#define DEF_EXT_BORDER_RELIEF	"flat"
-#define DEF_PLOT_BG_COLOR	ANTIQUEWHITE1
-#define DEF_PLOT_BG_MONO	WHITE
-#define DEF_INT_RELIEF_COLOR	"sunken"
-#define DEF_INT_RELIEF_MONO	"flat"
-#define DEF_INT_BW_MONO       	(char *)NULL
-#define DEF_INT_BW_COLOR      	"2"
-#define DEF_GRAPH_INVERT_XY	"false"
 
 static Tk_ConfigSpec configSpecs[] =
 {
@@ -99,15 +100,19 @@ static Tk_ConfigSpec configSpecs[] =
 	DEF_GRAPH_BG_MONO, Tk_Offset(Graph, border),
 	TK_CONFIG_MONO_ONLY | ALL_MASK},
     {TK_CONFIG_DOUBLE, "-barwidth", "barWidth", "BarWidth",
-	DEF_BAR_WIDTH, Tk_Offset(Graph, barWidth), BARCHART_MASK},
+	DEF_GRAPH_BAR_WIDTH, Tk_Offset(Graph, barWidth), BARCHART_MASK},
     {TK_CONFIG_SYNONYM, "-bd", "borderWidth", (char *)NULL, (char *)NULL, 0,
 	ALL_MASK},
     {TK_CONFIG_SYNONYM, "-bg", "background", (char *)NULL, (char *)NULL, 0,
 	ALL_MASK},
     {TK_CONFIG_PIXELS, "-borderwidth", "borderWidth", "BorderWidth",
-	(char *)NULL, Tk_Offset(Graph, extBWidth), ALL_MASK},
+	DEF_GRAPH_BORDER_WIDTH, Tk_Offset(Graph, extBW),
+	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_PIXELS, "-bottommargin", "bottomMargin", "Margin",
-	(char *)NULL, Tk_Offset(Graph, bottomMargin), ALL_MASK},
+	DEF_GRAPH_MARGIN, Tk_Offset(Graph, bottomMargin), ALL_MASK},
+    {TK_CONFIG_BOOLEAN, "-bufferelements", "bufferElements", "BufferElements",
+	DEF_GRAPH_BUFFERED, Tk_Offset(Graph, buffered),
+	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_ACTIVE_CURSOR, "-cursor", "cursor", "Cursor",
 	DEF_GRAPH_CURSOR, Tk_Offset(Graph, cursor),
 	ALL_MASK | TK_CONFIG_NULL_OK},
@@ -121,74 +126,79 @@ static Tk_ConfigSpec configSpecs[] =
     {TK_CONFIG_COLOR, "-foreground", "foreground", "Foreground",
 	DEF_GRAPH_FG_MONO, Tk_Offset(Graph, fgColorPtr),
 	TK_CONFIG_MONO_ONLY | ALL_MASK},
+    {TK_CONFIG_PIXELS, "-halo", "halo", "Halo",
+	DEF_GRAPH_HALO_BAR, Tk_Offset(Graph, halo), BARCHART_MASK},
     {TK_CONFIG_PIXELS, "-halo", "halo", "Halo",
 	DEF_GRAPH_HALO, Tk_Offset(Graph, halo), XYGRAPH_MASK},
     {TK_CONFIG_PIXELS, "-height", "height", "Height",
-	DEF_GRAPH_HEIGHT, Tk_Offset(Graph, reqHeight), ALL_MASK},
+	DEF_GRAPH_HEIGHT, Tk_Offset(Graph, reqHeight),
+	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_BOOLEAN, "-invertxy", "invertXY", "InvertXY",
 	DEF_GRAPH_INVERT_XY, Tk_Offset(Graph, inverted),
 	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_PIXELS, "-leftmargin", "leftMargin", "Margin",
-	(char *)NULL, Tk_Offset(Graph, leftMargin), ALL_MASK},
+	DEF_GRAPH_MARGIN, Tk_Offset(Graph, leftMargin),
+	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_COLOR, "-plotbackground", "plotBackground", "Background",
-	DEF_PLOT_BG_MONO, Tk_Offset(Graph, intBgColorPtr),
+	DEF_GRAPH_PLOT_BG_MONO, Tk_Offset(Graph, intBgColorPtr),
 	TK_CONFIG_MONO_ONLY | ALL_MASK},
     {TK_CONFIG_COLOR, "-plotbackground", "plotBackground", "Background",
-	DEF_PLOT_BG_COLOR, Tk_Offset(Graph, intBgColorPtr),
+	DEF_GRAPH_PLOT_BG_COLOR, Tk_Offset(Graph, intBgColorPtr),
 	TK_CONFIG_COLOR_ONLY | ALL_MASK},
     {TK_CONFIG_INT, "-plotborderwidth", "plotBorderWidth", "BorderWidth",
-	DEF_INT_BW_COLOR, Tk_Offset(Graph, intBWidth),
+	DEF_GRAPH_PLOT_BW_COLOR, Tk_Offset(Graph, intBW),
 	ALL_MASK | TK_CONFIG_COLOR_ONLY},
     {TK_CONFIG_INT, "-plotborderwidth", "plotBorderWidth", "BorderWidth",
-	(char *)NULL, Tk_Offset(Graph, intBWidth),
+	DEF_GRAPH_PLOT_BW_MONO, Tk_Offset(Graph, intBW),
 	ALL_MASK | TK_CONFIG_MONO_ONLY},
     {TK_CONFIG_RELIEF, "-plotrelief", "plotRelief", "Relief",
-	DEF_INT_RELIEF_MONO, Tk_Offset(Graph, intRelief),
-	TK_CONFIG_MONO_ONLY | TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
-    {TK_CONFIG_RELIEF, "-plotrelief", "plotRelief", "Relief",
-	DEF_INT_RELIEF_COLOR, Tk_Offset(Graph, intRelief),
-	TK_CONFIG_COLOR_ONLY | ALL_MASK},
+	DEF_GRAPH_PLOT_RELIEF, Tk_Offset(Graph, intRelief),
+	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_RELIEF, "-relief", "relief", "Relief",
-	DEF_EXT_BORDER_RELIEF, Tk_Offset(Graph, extRelief), ALL_MASK},
+	DEF_GRAPH_RELIEF, Tk_Offset(Graph, extRelief),
+	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_PIXELS, "-rightmargin", "rightMargin", "Margin",
-	(char *)NULL, Tk_Offset(Graph, rightMargin), ALL_MASK},
+	DEF_GRAPH_MARGIN, Tk_Offset(Graph, rightMargin),
+	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_STRING, "-title", "title", "Title",
-	(char *)NULL, Tk_Offset(Graph, title),
+	DEF_GRAPH_TITLE, Tk_Offset(Graph, title),
 	ALL_MASK | TK_CONFIG_NULL_OK},
     {TK_CONFIG_PIXELS, "-topmargin", "topMargin", "Margin",
-	(char *)NULL, Tk_Offset(Graph, topMargin), ALL_MASK},
+	DEF_GRAPH_MARGIN, Tk_Offset(Graph, topMargin),
+	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_PIXELS, "-width", "width", "Width",
-	DEF_GRAPH_WIDTH, Tk_Offset(Graph, reqWidth), ALL_MASK},
+	DEF_GRAPH_WIDTH, Tk_Offset(Graph, reqWidth),
+	TK_CONFIG_DONT_SET_DEFAULT | ALL_MASK},
     {TK_CONFIG_END, NULL, NULL, NULL, NULL, 0, 0}
 };
 
 double Blt_negInfinity, Blt_posInfinity;
 
-extern int Blt_CreateAxis _ANSI_ARGS_((Graph *, AxisType, int));
-extern int Blt_AxisCmd _ANSI_ARGS_((Graph *, GraphAxis *, int, char **, int));
-extern void Blt_ComputeAxes _ANSI_ARGS_((Graph *));
-extern void Blt_UpdateAxisBackgrounds _ANSI_ARGS_((Graph *, XColor *));
-extern int Blt_ComputeLayout _ANSI_ARGS_((Graph *));
-
-extern int Blt_LegendCmd _ANSI_ARGS_((Graph *, int, char **));
-extern int Blt_CreateLegend _ANSI_ARGS_((Graph *));
-
-extern int Blt_CreatePostScript _ANSI_ARGS_((Graph *));
-
-extern int Blt_CreateCrosshairs _ANSI_ARGS_((Graph *));
-extern int Blt_CrosshairsCmd _ANSI_ARGS_((Graph *, int, char **));
-
-static void DisplayGraph _ANSI_ARGS_((ClientData));
-static void DestroyGraph _ANSI_ARGS_((ClientData));
-static int GraphWidgetCmd _ANSI_ARGS_((ClientData, Tcl_Interp *, int,
-	char **));
-extern int Blt_TagCmd _ANSI_ARGS_((Graph *, int, char **));
-extern int Blt_ElementCmd _ANSI_ARGS_((Graph *, int, char **));
+extern int Blt_CreateAxis _ANSI_ARGS_((Graph *graphPtr, AxisType type, int));
+extern int Blt_AxisCmd _ANSI_ARGS_((Graph *graphPtr, GraphAxis *axisPtr,
+	int argc, char **argv, int flags));
+extern void Blt_ComputeAxes _ANSI_ARGS_((Graph *graphPtr));
+extern void Blt_UpdateAxisBackgrounds _ANSI_ARGS_((Graph *graphPtr,
+	XColor *bgColorPtr));
+extern int Blt_ComputeLayout _ANSI_ARGS_((Graph *graphPtr));
+extern int Blt_LegendCmd _ANSI_ARGS_((Graph *graphPtr, int argc, char **argv));
+extern int Blt_CreateLegend _ANSI_ARGS_((Graph *graphPtr));
+extern int Blt_CreatePostScript _ANSI_ARGS_((Graph *graphPtr));
+extern int Blt_CreateCrosshairs _ANSI_ARGS_((Graph *graphPtr));
+extern int Blt_CrosshairsCmd _ANSI_ARGS_((Graph *graphPtr, int argc,
+	char **argv));
+static void DisplayGraph _ANSI_ARGS_((ClientData clientData));
+static void DestroyGraph _ANSI_ARGS_((ClientData clientData));
+static int GraphWidgetCmd _ANSI_ARGS_((ClientData clientData,
+	Tcl_Interp *interp, int argc, char **argv));
+extern int Blt_TagCmd _ANSI_ARGS_((Graph *graphPtr, int argc, char **argv));
+extern int Blt_ElementCmd _ANSI_ARGS_((Graph *graphPtr, int argc,
+	char **argv));
 
 /*
  *--------------------------------------------------------------
  *
- * Blt_EventuallyRedraw --
+ * Blt_RedrawGraph --
  *
  *	Tell the Tk dispatcher to call the graph display routine at
  *	the next idle point.  This request is made only if the window
@@ -203,7 +213,7 @@ extern int Blt_ElementCmd _ANSI_ARGS_((Graph *, int, char **));
  *--------------------------------------------------------------
  */
 void
-Blt_EventuallyRedraw(graphPtr)
+Blt_RedrawGraph(graphPtr)
     Graph *graphPtr;		/* Graph widget record */
 {
     if (Tk_IsMapped(graphPtr->tkwin) && !(graphPtr->flags & REDRAW_PENDING)) {
@@ -238,8 +248,8 @@ GraphEventProc(clientData, eventPtr)
 
     if (eventPtr->type == Expose) {
 	if (eventPtr->xexpose.count == 0) {
-	    graphPtr->flags |= REDRAW_ALL;
-	    Blt_EventuallyRedraw(graphPtr);
+	    graphPtr->flags |= REFRESH;
+	    Blt_RedrawGraph(graphPtr);
 	}
     } else if (eventPtr->type == DestroyNotify) {
 	Tcl_DeleteCommand(graphPtr->interp, Tk_PathName(graphPtr->tkwin));
@@ -249,8 +259,8 @@ GraphEventProc(clientData, eventPtr)
 	}
 	Tk_EventuallyFree((ClientData)graphPtr, DestroyGraph);
     } else if (eventPtr->type == ConfigureNotify) {
-	graphPtr->flags |= (LAYOUT_PENDING | LAYOUT_ALL | REDRAW_ALL);
-	Blt_EventuallyRedraw(graphPtr);
+	graphPtr->flags |= (LAYOUT_NEEDED | LAYOUT_ALL | REFRESH);
+	Blt_RedrawGraph(graphPtr);
     }
 }
 
@@ -447,7 +457,6 @@ DestroyGraph(clientData)
 	axisPtr = graphPtr->axisArr[i];
 	(*axisPtr->destroyProc) (graphPtr, axisPtr);
     }
-
     (*graphPtr->legendPtr->destroyProc) (graphPtr);
     (*graphPtr->postscript->destroyProc) (graphPtr);
     (*graphPtr->crosshairs->destroyProc) (graphPtr);
@@ -459,6 +468,9 @@ DestroyGraph(clientData)
     if (graphPtr->extBgGC != NULL) {
 	Tk_FreeGC(graphPtr->display, graphPtr->extBgGC);
     }
+    if (graphPtr->elemMap != None) {
+	XFreePixmap(graphPtr->display, graphPtr->elemMap);
+    }
     Tk_FreeOptions(configSpecs, (char *)graphPtr, graphPtr->display,
 	configFlags[graphPtr->type]);
     free((char *)graphPtr);
@@ -512,6 +524,11 @@ CreateGraph(interp, parent, pathName, type)
     graphPtr->display = Tk_Display(tkwin);
     graphPtr->interp = interp;
     graphPtr->type = type;
+    graphPtr->width = graphPtr->height = 400;
+    graphPtr->reqWidth = graphPtr->reqHeight = 400;
+    graphPtr->buffered = 1;
+    graphPtr->intRelief = TK_RELIEF_SUNKEN;
+    graphPtr->extRelief = TK_RELIEF_FLAT;
 
     Tcl_InitHashTable(&(graphPtr->elemTable), TCL_STRING_KEYS);
     Blt_InitLinkedList(&(graphPtr->elemList), TCL_STRING_KEYS);
@@ -589,9 +606,7 @@ ConfigureGraph(graphPtr, argc, argv, flags)
     GC newGC;
     XGCValues gcValues;
     unsigned long gcMask;
-    int oldFlag;
 
-    oldFlag = graphPtr->inverted;
     if (Tk_ConfigureWidget(graphPtr->interp, graphPtr->tkwin, configSpecs,
 	    argc, argv, (char *)graphPtr, flags) != TCL_OK) {
 	return TCL_ERROR;
@@ -604,7 +619,7 @@ ConfigureGraph(graphPtr, argc, argv, flags)
     }
     Tk_GeometryRequest(graphPtr->tkwin, graphPtr->reqWidth,
 	graphPtr->reqHeight);
-    Tk_SetInternalBorder(graphPtr->tkwin, graphPtr->extBWidth);
+    Tk_SetInternalBorder(graphPtr->tkwin, graphPtr->extBW);
     Tk_SetBackgroundFromBorder(graphPtr->tkwin, graphPtr->border);
 
     borderColor = Tk_3DBorderColor(graphPtr->border);
@@ -638,20 +653,46 @@ ConfigureGraph(graphPtr, argc, argv, flags)
 	Tk_FreeGC(graphPtr->display, graphPtr->intGC);
     }
     graphPtr->intGC = newGC;
-
-    if (oldFlag != graphPtr->inverted) {
+    /*
+     * If the -inverted option changed, we need to readjust the pointers
+     * to the axes and recompute the their scales.
+     */
+    if (Blt_OptionChanged(configSpecs, "-invertxy", (char *)NULL)) {
 	AdjustAxisPointers(graphPtr);
 	Blt_ComputeAxes(graphPtr);
     }
+    /*
+     * Free the pixmap if we're not buffering the display of elements
+     * anymore.
+     */
+    if ((!graphPtr->buffered) && (graphPtr->elemMap != None)) {
+	XFreePixmap(graphPtr->display, graphPtr->elemMap);
+	graphPtr->elemMap = None;
+    }
     /*
      * Reconfigure the crosshairs in case the plotting area's
      * background color changed
      */
     (*graphPtr->crosshairs->configProc) (graphPtr);
 
-    /* Title font, title strings, or margins may have changed layout */
-    graphPtr->flags |= (REDRAW_ALL | LAYOUT_PENDING | LAYOUT_ALL);
-    Blt_EventuallyRedraw(graphPtr);
+    /*
+     *  Update the layout of the graph (and redraw the elements) if
+     *  any of the following graph options which affect the size of
+     *	the plotting area has changed.
+     *
+     *      -borderwidth, -plotborderwidth
+     *	    -font, -title
+     *	    -width, -height
+     *	    -invertxy
+     *	    -bottommargin, -leftmargin, -rightmargin, -topmargin
+     *
+     */
+    if (Blt_OptionChanged(configSpecs, "-invertxy", "-title", "-font",
+	    "-*margin", "-*width", "-height", (char *)NULL)) {
+	graphPtr->flags |= (DIRTY | LAYOUT_NEEDED | LAYOUT_ALL);
+    }
+    graphPtr->flags |= REFRESH;
+    Blt_RedrawGraph(graphPtr);
     return TCL_OK;
 }
 
@@ -699,8 +740,8 @@ GraphCmd(clientData, interp, argc, argv)
 	    "\"", (char *)NULL);
 	return TCL_ERROR;
     }
-    Blt_negInfinity = (double)MIN_DBL_VALUE;
-    Blt_posInfinity = (double)MAX_DBL_VALUE;
+    Blt_negInfinity = (double)MAX_NEG_VAL;
+    Blt_posInfinity = (double)MAX_POS_VAL;
     graphPtr = CreateGraph(interp, tkwin, argv[1], type);
     if (graphPtr == NULL) {
 	return TCL_ERROR;
@@ -862,9 +903,9 @@ Blt_LayoutGraph(graphPtr)
 	entryPtr != NULL; entryPtr = Blt_NextListEntry(entryPtr)) {
 	elemPtr = (Element *)Blt_GetListValue(entryPtr);
 	if ((graphPtr->flags & LAYOUT_ALL) ||
-	    (elemPtr->flags & LAYOUT_PENDING)) {
+	    (elemPtr->flags & LAYOUT_NEEDED)) {
 	    (*elemPtr->layoutProc) (graphPtr, elemPtr);
-	    elemPtr->flags &= ~LAYOUT_PENDING;
+	    elemPtr->flags &= ~LAYOUT_NEEDED;
 	}
     }
 
@@ -873,12 +914,12 @@ Blt_LayoutGraph(graphPtr)
 	entryPtr != NULL; entryPtr = Blt_NextListEntry(entryPtr)) {
 	tagPtr = (Tag *)Blt_GetListValue(entryPtr);
 	if ((graphPtr->flags & LAYOUT_ALL) ||
-	    (tagPtr->flags & LAYOUT_PENDING)) {
+	    (tagPtr->flags & LAYOUT_NEEDED)) {
 	    (*tagPtr->layoutProc) (graphPtr, tagPtr);
-	    tagPtr->flags &= ~LAYOUT_PENDING;
+	    tagPtr->flags &= ~LAYOUT_NEEDED;
 	}
     }
-    graphPtr->flags &= ~(LAYOUT_ALL | LAYOUT_PENDING);
+    graphPtr->flags &= ~(LAYOUT_ALL | LAYOUT_NEEDED);
 }
 
 /*
@@ -908,7 +949,40 @@ DisplayElements(graphPtr)
     for (entryPtr = Blt_FirstListEntry(&(graphPtr->elemList));
 	entryPtr != NULL; entryPtr = Blt_NextListEntry(entryPtr)) {
 	elemPtr = (Element *)Blt_GetListValue(entryPtr);
-	(*elemPtr->displayProc) (graphPtr, elemPtr);
+	(*elemPtr->displayProc) (graphPtr, elemPtr, ELEM_NORMAL);
+    }
+}
+
+/*
+ * -----------------------------------------------------------------
+ *
+ * DisplayActiveElements --
+ *
+ *	Calls the individual element drawing routines to display
+ *	the active colors for each element.
+ *
+ * Results:
+ *	None
+ *
+ * Side Effects:
+ *	Elements are drawn into the drawable (pixmap) which will
+ *	eventually be displayed in the graph window.
+ *
+ * -----------------------------------------------------------------
+ */
+static void
+DisplayActiveElements(graphPtr)
+    Graph *graphPtr;
+{
+    Blt_ListEntry *entryPtr;
+    register Element *elemPtr;
+
+    for (entryPtr = Blt_FirstListEntry(&(graphPtr->elemList));
+	entryPtr != NULL; entryPtr = Blt_NextListEntry(entryPtr)) {
+	elemPtr = (Element *)Blt_GetListValue(entryPtr);
+	if (elemPtr->flags & ACTIVE) {
+	    (*elemPtr->displayProc) (graphPtr, elemPtr, ELEM_ACTIVE);
+	}
     }
 }
 
@@ -942,7 +1016,7 @@ DisplayTags(graphPtr)
     for (listPtr = Blt_FirstListEntry(&(graphPtr->tagList)); listPtr != NULL;
 	listPtr = Blt_NextListEntry(listPtr)) {
 	tagPtr = (Tag *)Blt_GetListValue(listPtr);
-	mapped = TRUE;
+	mapped = 1;
 
 	if (tagPtr->elemId != NULL) {
 	    Tcl_HashEntry *entryPtr;
@@ -994,7 +1068,7 @@ DisplayExterior(graphPtr, rectPtr)
     int x, y;
     XRectangle rectArr[4];
     GraphAxis *axisPtr;
-    TextAttr textAttr;
+    TextAttributes textAttr;
     register int i;
 
     /*
@@ -1016,11 +1090,12 @@ DisplayExterior(graphPtr, rectPtr)
 	rectArr, 4);
 
     /* Interior 3D border */
+
     if ((graphPtr->intRelief != TK_RELIEF_FLAT) &&
-	(graphPtr->intBWidth > 0)) {
+	(graphPtr->intBW > 0)) {
 	Tk_Draw3DRectangle(graphPtr->display, graphPtr->canvas,
 	    graphPtr->border, rectPtr->x, rectPtr->y, rectPtr->width,
-	    rectPtr->height, graphPtr->intBWidth, graphPtr->intRelief);
+	    rectPtr->height, graphPtr->intBW, graphPtr->intRelief);
     }
     /* Legend */
     if (graphPtr->legendPtr->useDefault) {
@@ -1036,7 +1111,7 @@ DisplayExterior(graphPtr, rectPtr)
 
     if (graphPtr->title != NULL) {
 	x = (graphPtr->extreme.x + graphPtr->origin.x) / 2;
-	y = graphPtr->extBWidth + TEXTHEIGHT(graphPtr->fontPtr);
+	y = graphPtr->extBW + TEXTHEIGHT(graphPtr->fontPtr);
 	Blt_DrawText(graphPtr->display, graphPtr->canvas, graphPtr->title,
 	    &textAttr, x, y);
     }
@@ -1049,10 +1124,10 @@ DisplayExterior(graphPtr, rectPtr)
     }
     /* Exterior 3D border */
     if ((graphPtr->extRelief != TK_RELIEF_FLAT) &&
-	(graphPtr->extBWidth > 0)) {
+	(graphPtr->extBW > 0)) {
 	Tk_Draw3DRectangle(graphPtr->display, graphPtr->canvas,
 	    graphPtr->border, 0, 0, graphPtr->width, graphPtr->height,
-	    graphPtr->extBWidth, graphPtr->extRelief);
+	    graphPtr->extBW, graphPtr->extRelief);
     }
 }
 
@@ -1077,7 +1152,7 @@ DisplayGraph(clientData)
     ClientData clientData;
 {
     register Graph *graphPtr = (Graph *)clientData;
-    XRectangle plotRect;
+    XRectangle plotArea;
     int twiceBW;
     GraphLegend *legendPtr;
 
@@ -1096,7 +1171,7 @@ DisplayGraph(clientData)
     graphPtr->width = Tk_Width(graphPtr->tkwin);
     graphPtr->height = Tk_Height(graphPtr->tkwin);
 
-    if (graphPtr->flags & LAYOUT_PENDING) {
+    if (graphPtr->flags & LAYOUT_NEEDED) {
 	if (graphPtr->flags & LAYOUT_ALL) {
 	    if (Blt_ComputeLayout(graphPtr) != TCL_OK) {
 		return;		/* Not enough room to plot graph */
@@ -1104,61 +1179,96 @@ DisplayGraph(clientData)
 	}
 	Blt_LayoutGraph(graphPtr);
     }
-    twiceBW = (graphPtr->intBWidth * 2);
-
-    /* Determine the plotting surface region of the graph window */
-    plotRect.x = graphPtr->origin.x - graphPtr->intBWidth;
-    plotRect.y = graphPtr->extreme.y - graphPtr->intBWidth;
-    plotRect.width = (graphPtr->extreme.x - graphPtr->origin.x) + twiceBW;
-    plotRect.height = (graphPtr->origin.y - graphPtr->extreme.y) + twiceBW;
+    /*
+     * Determine the plotting surface region of the graph window
+     */
+    twiceBW = (graphPtr->intBW * 2);
+    plotArea.x = graphPtr->origin.x - graphPtr->intBW;
+    plotArea.y = graphPtr->extreme.y - graphPtr->intBW;
+    plotArea.width = (graphPtr->extreme.x - graphPtr->origin.x) + twiceBW;
+    plotArea.height = (graphPtr->origin.y - graphPtr->extreme.y) + twiceBW;
 
     /*
-     * Create a canvas the size of the window, filling it with the
-     * plotting area's background color
+     * Create a pixmap (for double buffering) the size of the window
      */
     graphPtr->canvas = XCreatePixmap(graphPtr->display,
 	Tk_WindowId(graphPtr->tkwin), graphPtr->width, graphPtr->height,
 	Tk_Depth(graphPtr->tkwin));
-    XFillRectangles(graphPtr->display, graphPtr->canvas, graphPtr->intGC,
-	&plotRect, 1);
+
+    if (graphPtr->buffered) {
+	if ((graphPtr->elemWidth != graphPtr->width) ||
+	    (graphPtr->elemHeight != graphPtr->height) ||
+	    (graphPtr->elemMap == None)) {
+	    /*
+	     * Create a pixmap to save elements if one doesn't already
+	     * exist or the size of the window has changed.
+	     */
+	    graphPtr->elemWidth = graphPtr->width;
+	    graphPtr->elemHeight = graphPtr->height;
+	    if (graphPtr->elemMap != None) {
+		XFreePixmap(graphPtr->display, graphPtr->elemMap);
+	    }
+	    graphPtr->elemMap = XCreatePixmap(graphPtr->display,
+		Tk_WindowId(graphPtr->tkwin), graphPtr->elemWidth,
+		graphPtr->elemHeight, Tk_Depth(graphPtr->tkwin));
+	    graphPtr->flags |= DIRTY;
+	}
+	if (graphPtr->flags & DIRTY) {
+	    Pixmap save;
+	    /*
+	     * Clear the background and draw the elements into the pixmap.
+	     */
+	    XFillRectangles(graphPtr->display, graphPtr->elemMap,
+		graphPtr->intGC, &plotArea, 1);
+	    save = graphPtr->canvas;
+	    graphPtr->canvas = graphPtr->elemMap;
+	    DisplayElements(graphPtr);
+	    graphPtr->canvas = save;
+	    graphPtr->flags &= ~DIRTY;
+	}
+	XCopyArea(graphPtr->display, graphPtr->elemMap, graphPtr->canvas,
+	    graphPtr->extGC, plotArea.x, plotArea.y,
+	    (unsigned int)plotArea.width, (unsigned int)plotArea.height,
+	    plotArea.x, plotArea.y);
+    } else {
+	XFillRectangles(graphPtr->display, graphPtr->canvas, graphPtr->intGC,
+	    &plotArea, 1);
+	DisplayElements(graphPtr);
+    }
+    DisplayTags(graphPtr);
+
     legendPtr = graphPtr->legendPtr;
     if (!legendPtr->useDefault) {
 	(*legendPtr->displayProc) (graphPtr);
     }
-    /* Draw the graph elements and tags on the canvas */
-    if (graphPtr->type == GRAPH) {
-	DisplayTags(graphPtr);
-	DisplayElements(graphPtr);
-    } else {
-	DisplayElements(graphPtr);
-	DisplayTags(graphPtr);
-    }
+    DisplayActiveElements(graphPtr);
 
-    /* Disable crosshairs and update lengths before drawing */
+    /* Disable crosshairs and update lengths before drawing to the display */
     (*graphPtr->crosshairs->toggleProc) (graphPtr);
     (*graphPtr->crosshairs->updateProc) (graphPtr);
 
-    if (graphPtr->flags & REDRAW_ALL) {
-	DisplayExterior(graphPtr, &plotRect);
+    if (graphPtr->flags & REFRESH) {
+	DisplayExterior(graphPtr, &plotArea);
 	XCopyArea(graphPtr->display, graphPtr->canvas,
 	    Tk_WindowId(graphPtr->tkwin), graphPtr->extGC, 0, 0,
 	    graphPtr->width, graphPtr->height, 0, 0);
     } else {
 	/* Draw just the interior plotting region */
 
-	plotRect.x += graphPtr->intBWidth;
-	plotRect.y += graphPtr->intBWidth;
-	plotRect.width -= (graphPtr->intBWidth * 2);
-	plotRect.height -= (graphPtr->intBWidth * 2);
+	plotArea.x += graphPtr->intBW;
+	plotArea.y += graphPtr->intBW;
+	plotArea.width -= (graphPtr->intBW * 2);
+	plotArea.height -= (graphPtr->intBW * 2);
 	XCopyArea(graphPtr->display, graphPtr->canvas,
-	    Tk_WindowId(graphPtr->tkwin), graphPtr->extGC, plotRect.x,
-	    plotRect.y, (unsigned int)plotRect.width,
-	    (unsigned int)plotRect.height, plotRect.x, plotRect.y);
+	    Tk_WindowId(graphPtr->tkwin), graphPtr->extGC, plotArea.x,
+	    plotArea.y, (unsigned int)plotArea.width,
+	    (unsigned int)plotArea.height, plotArea.x, plotArea.y);
     }
     (*graphPtr->crosshairs->toggleProc) (graphPtr);
 
     XFreePixmap(graphPtr->display, graphPtr->canvas);
-    graphPtr->flags &= ~REDRAW_ALL;
+    graphPtr->canvas = None;
+    graphPtr->flags &= ~REFRESH;
 }
 
 int
@@ -1197,7 +1307,8 @@ Blt_BarchartInit(interp)
     }
     tkwin = Tk_MainWindow(interp);
     if (tkwin == NULL) {
-	Tcl_AppendResult(interp, "\"blt_barchart\" requires Tk", (char *)NULL);
+	Tcl_AppendResult(interp, "\"blt_barchart\" requires Tk",
+	    (char *)NULL);
 	return TCL_ERROR;
     }
     Tcl_SetVar2(interp, "blt_versions", "blt_barchart", GRAPH_VERSION,
