@@ -70,8 +70,9 @@ Blt_FindCmd(interp, cmdName, clientDataPtr)
  *
  * Blt_OptionChanged --
  *
- *      Given the specs and offsets of a Tk_ConfigSpec structure
- *      indicates if any of the requested options have changed.
+ *      Given the specs and option argument names (terminated by a NULL),
+ *	search  a Tk_ConfigSpec structure for any matching specs which
+ *      has been reset.
  *
  * Results:
  *      Returns 1 if one of the options has changed, 0 otherwise.
@@ -80,17 +81,31 @@ Blt_FindCmd(interp, cmdName, clientDataPtr)
  */
 
 int
-Blt_OptionChanged(specs, offsets, numOffsets)
+Blt_OptionChanged(specs, opt0, opt1, opt2, opt3, opt4, opt5, opt6, opt7,
+    opt8, opt9, opt10)
     Tk_ConfigSpec specs[];
-    int offsets[];
-    int numOffsets;
+    char *opt0, *opt1, *opt2, *opt3, *opt4, *opt5, *opt6, *opt7, *opt8;
+    char *opt9, *opt10;
 {
     register Tk_ConfigSpec *specPtr;
-    register int i;
+    char *argv[12];
+    register char **argPtr;
 
+    argv[0] = opt0;
+    argv[1] = opt1;
+    argv[2] = opt2;
+    argv[3] = opt3;
+    argv[4] = opt4;
+    argv[5] = opt5;
+    argv[6] = opt6;
+    argv[7] = opt7;
+    argv[8] = opt8;
+    argv[9] = opt9;
+    argv[10] = opt10;
+    argv[11] = NULL;
     for (specPtr = specs; specPtr->type != TK_CONFIG_END; specPtr++) {
-	for (i = 0; i < numOffsets; i++) {
-	    if ((offsets[i] == specPtr->offset) &&
+	for (argPtr = argv; *argPtr != NULL; argPtr++) {
+	    if ((Tcl_StringMatch(specPtr->argvName, *argPtr)) &&
 		(specPtr->specFlags & TK_CONFIG_OPTION_SPECIFIED)) {
 		return 1;
 	    }
